@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,20 +18,29 @@
  */
 package org.apache.bookkeeper.mledger.impl;
 
+import static org.apache.bookkeeper.mledger.impl.ManagedCursorImpl.CURSOR_INTERNAL_PROPERTY_PREFIX;
+import static org.apache.bookkeeper.mledger.util.Futures.executeWithRetry;
 import static org.testng.Assert.assertEquals;
-
+import static org.testng.Assert.assertNull;
+import static org.testng.Assert.assertTrue;
+import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
-
 import org.apache.bookkeeper.mledger.ManagedCursor;
 import org.apache.bookkeeper.mledger.ManagedLedger;
 import org.apache.bookkeeper.mledger.ManagedLedgerConfig;
+import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactory;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.test.MockedBookKeeperTestCase;
 import org.apache.pulsar.common.api.proto.CommandSubscribe.InitialPosition;
+import org.apache.pulsar.common.util.FutureUtil;
+import org.testng.Assert;
 import org.testng.annotations.Test;
 
 public class ManagedCursorPropertiesTest extends MockedBookKeeperTestCase {
@@ -209,6 +218,12 @@ public class ManagedCursorPropertiesTest extends MockedBookKeeperTestCase {
         assertEquals(c1.getProperties(), properties);
         assertEquals(c1.getCursorProperties(), cursorPropertiesUpdated);
 
+        c1.putCursorProperty("custom3", "Five").get();
+        cursorPropertiesUpdated.put("custom3", "Five");
+        c1.removeCursorProperty("custom1").get();
+        cursorPropertiesUpdated.remove("custom1");
+        assertEquals(c1.getCursorProperties(), cursorPropertiesUpdated);
+
         // Create a new factory to force a managed ledger close and recovery
         ManagedLedgerFactory factory2 = new ManagedLedgerFactoryImpl(metadataStore, bkc);
         // Reopen the managed ledger
@@ -218,6 +233,64 @@ public class ManagedCursorPropertiesTest extends MockedBookKeeperTestCase {
         assertEquals(c1.getProperties(), properties);
         assertEquals(c1.getCursorProperties(), cursorPropertiesUpdated);
 
+        ledger.close();
+
         factory2.shutdown();
+
+        // Create a new factory to force a managed ledger close and recovery
+        ManagedLedgerFactory factory3 = new ManagedLedgerFactoryImpl(metadataStore, bkc);
+        // Reopen the managed ledger
+        ledger = factory3.open("testUpdateCursorProperties", new ManagedLedgerConfig());
+        c1 = ledger.openCursor("c1");
+
+        c1.putCursorProperty(CURSOR_INTERNAL_PROPERTY_PREFIX + "test", "test").get(10, TimeUnit.SECONDS);
+        c1.putCursorProperty("custom4", "custom4").get(10, TimeUnit.SECONDS);
+        c1.setCursorProperties(cursorPropertiesUpdated).get(10, TimeUnit.SECONDS);
+
+        cursorPropertiesUpdated.put(CURSOR_INTERNAL_PROPERTY_PREFIX + "test", "test");
+
+        try {
+            c1.setCursorProperties(cursorPropertiesUpdated).get(10, TimeUnit.SECONDS);
+            Assert.fail("Should fail");
+        } catch (Exception e) {
+            assertTrue(
+                    FutureUtil.unwrapCompletionException(e).getMessage().contains("The property key can't start with"));
+        }
+
+        assertEquals(c1.getCursorProperties(), cursorPropertiesUpdated);
+
+        ledger.close();
+
+        factory3.shutdown();
+    }
+
+    @Test
+    public void testUpdateCursorPropertiesConcurrent() throws Exception {
+        ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig());
+        ManagedCursor c1 = ledger.openCursor("c1");
+
+        List<CompletableFuture<Void>> futures = new ArrayList<>();
+
+        Map<String, String> map = new HashMap<>();
+        map.put("a", "1");
+        map.put("b", "2");
+        map.put("c", "3");
+
+        futures.add(executeWithRetry(() -> c1.setCursorProperties(map),
+                ManagedLedgerException.BadVersionException.class, 3));
+
+        futures.add(executeWithRetry(() -> c1.putCursorProperty("a", "2"),
+                ManagedLedgerException.BadVersionException.class, 3));
+
+        futures.add(executeWithRetry(() -> c1.removeCursorProperty("c"),
+                ManagedLedgerException.BadVersionException.class, 3));
+
+        for (CompletableFuture<Void> future : futures) {
+            future.get(10, TimeUnit.SECONDS);
+        }
+
+        assertEquals(c1.getCursorProperties().get("a"), "2");
+        assertEquals(c1.getCursorProperties().get("b"), "2");
+        assertNull(c1.getCursorProperties().get("c"));
     }
 }
