@@ -27,14 +27,16 @@ import io.swagger.annotations.Example;
 import io.swagger.annotations.ExampleProperty;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
+import java.util.Set;
 import java.util.concurrent.CompletableFuture;
+import java.util.regex.Pattern;
 import java.util.stream.Collectors;
-import java.util.stream.Stream;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.GET;
 import javax.ws.rs.POST;
@@ -61,10 +63,14 @@ import org.apache.pulsar.common.policies.data.BrokerNamespaceIsolationData;
 import org.apache.pulsar.common.policies.data.BrokerNamespaceIsolationDataImpl;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.ClusterDataImpl;
+import org.apache.pulsar.common.policies.data.ClusterOperation;
 import org.apache.pulsar.common.policies.data.ClusterPolicies.ClusterUrl;
 import org.apache.pulsar.common.policies.data.ClusterPoliciesImpl;
 import org.apache.pulsar.common.policies.data.FailureDomainImpl;
 import org.apache.pulsar.common.policies.data.NamespaceIsolationDataImpl;
+import org.apache.pulsar.common.policies.data.NamespaceIsolationPolicyUnloadScope;
+import org.apache.pulsar.common.policies.data.PolicyName;
+import org.apache.pulsar.common.policies.data.PolicyOperation;
 import org.apache.pulsar.common.policies.impl.NamespaceIsolationPolicies;
 import org.apache.pulsar.common.policies.impl.NamespaceIsolationPolicyImpl;
 import org.apache.pulsar.common.util.FutureUtil;
@@ -114,7 +120,7 @@ public class ClustersBase extends AdminResource {
     public void getCluster(@Suspended AsyncResponse asyncResponse,
                            @ApiParam(value = "The cluster name", required = true)
                            @PathParam("cluster") String cluster) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.GET_CLUSTER)
                 .thenCompose(__ -> clusterResources().getClusterAsync(cluster))
                 .thenAccept(clusterData -> {
                     asyncResponse.resume(clusterData
@@ -159,7 +165,7 @@ public class ClustersBase extends AdminResource {
                 )
             )
         ) ClusterDataImpl clusterData) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.CREATE_CLUSTER)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> {
                     NamedEntity.checkName(cluster);
@@ -224,7 +230,7 @@ public class ClustersBase extends AdminResource {
                 )
             )
         ) ClusterDataImpl clusterData) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.UPDATE_CLUSTER)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> {
                     try {
@@ -268,7 +274,7 @@ public class ClustersBase extends AdminResource {
             required = true
         )
         @PathParam("cluster") String cluster) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.CLUSTER_MIGRATION, PolicyOperation.READ)
                 .thenCompose(__ -> clusterResources().getClusterPoliciesResources().getClusterPoliciesAsync(cluster))
                 .thenAccept(policies -> {
                     asyncResponse.resume(
@@ -323,7 +329,7 @@ public class ClustersBase extends AdminResource {
             asyncResponse.resume(new RestException(Status.BAD_REQUEST, "Cluster url must not be empty"));
             return;
         }
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.CLUSTER_MIGRATION, PolicyOperation.WRITE)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> clusterResources().getClusterPoliciesResources().setPoliciesWithCreateAsync(cluster,
                         old -> {
@@ -373,7 +379,7 @@ public class ClustersBase extends AdminResource {
                                                    "cluster-b"
                                                 ]""")))
                                     LinkedHashSet<String> peerClusterNames) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.UPDATE_PEER_CLUSTER)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> innerSetPeerClusterNamesAsync(cluster, peerClusterNames))
                 .thenAccept(__ -> {
@@ -434,7 +440,7 @@ public class ClustersBase extends AdminResource {
     public void getPeerCluster(@Suspended AsyncResponse asyncResponse,
                                @ApiParam(value = "The cluster name", required = true)
                                @PathParam("cluster") String cluster) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.GET_PEER_CLUSTER)
                 .thenCompose(__ -> clusterResources().getClusterAsync(cluster))
                 .thenAccept(clusterOpt -> {
                     ClusterData clusterData =
@@ -463,7 +469,7 @@ public class ClustersBase extends AdminResource {
     public void deleteCluster(@Suspended AsyncResponse asyncResponse,
                               @ApiParam(value = "The cluster name", required = true)
                               @PathParam("cluster") String cluster) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.DELETE_CLUSTER)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> internalDeleteClusterAsync(cluster))
                 .thenAccept(__ -> {
@@ -522,7 +528,7 @@ public class ClustersBase extends AdminResource {
         @Suspended AsyncResponse asyncResponse,
         @ApiParam(value = "The cluster name", required = true) @PathParam("cluster") String cluster
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.READ)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, Status.NOT_FOUND))
                 .thenCompose(__ -> internalGetNamespaceIsolationPolicies(cluster))
                 .thenAccept(asyncResponse::resume)
@@ -580,7 +586,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The name of the namespace isolation policy", required = true)
         @PathParam("policyName") String policyName
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.READ)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, Status.PRECONDITION_FAILED))
                 .thenCompose(__ -> internalGetNamespaceIsolationPolicies(cluster))
                 .thenAccept(policies -> {
@@ -616,7 +622,7 @@ public class ClustersBase extends AdminResource {
             @Suspended AsyncResponse asyncResponse,
             @ApiParam(value = "The cluster name", required = true)
             @PathParam("cluster") String cluster) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.READ)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, Status.PRECONDITION_FAILED))
                 .thenCompose(__ -> pulsar().getLoadManager().get().getAvailableBrokersAsync())
                 .thenCompose(availableBrokers -> internalGetNamespaceIsolationPolicies(cluster)
@@ -673,7 +679,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The broker name (<broker-hostname>:<web-service-port>)", required = true,
             example = "broker1:8080")
         @PathParam("broker") String broker) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.READ)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> internalGetNamespaceIsolationPolicies(cluster))
                 .thenApply(policies -> internalGetBrokerNsIsolationData(broker, policies))
@@ -708,7 +714,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The namespace isolation policy data", required = true)
         NamespaceIsolationDataImpl policyData
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.WRITE)
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> {
@@ -721,10 +727,13 @@ public class ClustersBase extends AdminResource {
                                         .setIsolationDataWithCreateAsync(cluster, (p) -> Collections.emptyMap())
                                         .thenApply(__ -> new NamespaceIsolationPolicies()))
                 ).thenCompose(nsIsolationPolicies -> {
+                    NamespaceIsolationDataImpl oldPolicy = nsIsolationPolicies
+                            .getPolicies().getOrDefault(policyName, null);
                     nsIsolationPolicies.setPolicy(policyName, policyData);
                     return namespaceIsolationPolicies()
-                                    .setIsolationDataAsync(cluster, old -> nsIsolationPolicies.getPolicies());
-                }).thenCompose(__ -> filterAndUnloadMatchedNamespaceAsync(policyData))
+                            .setIsolationDataAsync(cluster, old -> nsIsolationPolicies.getPolicies())
+                            .thenApply(__ -> oldPolicy);
+                }).thenCompose(oldPolicy -> filterAndUnloadMatchedNamespaceAsync(cluster, policyData, oldPolicy))
                 .thenAccept(__ -> {
                     log.info("[{}] Successful to update clusters/{}/namespaceIsolationPolicies/{}.",
                             clientAppId(), cluster, policyName);
@@ -758,42 +767,95 @@ public class ClustersBase extends AdminResource {
     /**
      * Get matched namespaces; call unload for each namespaces.
      */
-    private CompletableFuture<Void> filterAndUnloadMatchedNamespaceAsync(NamespaceIsolationDataImpl policyData) {
+    private CompletableFuture<Void> filterAndUnloadMatchedNamespaceAsync(String cluster,
+                                                                         NamespaceIsolationDataImpl policyData,
+                                                                         NamespaceIsolationDataImpl oldPolicy) {
+        // exit early if none of the namespaces need to be unloaded
+        if (NamespaceIsolationPolicyUnloadScope.none.equals(policyData.getUnloadScope())) {
+            return CompletableFuture.completedFuture(null);
+        }
+
         PulsarAdmin adminClient;
         try {
             adminClient = pulsar().getAdminClient();
         } catch (PulsarServerException e) {
             return FutureUtil.failedFuture(e);
         }
-        return adminClient.tenants().getTenantsAsync()
-                .thenCompose(tenants -> {
-                    Stream<CompletableFuture<List<String>>> completableFutureStream = tenants.stream()
-                            .map(tenant -> adminClient.namespaces().getNamespacesAsync(tenant));
-                    return FutureUtil.waitForAll(completableFutureStream)
-                            .thenApply(namespaces -> {
-                                // if namespace match any policy regex, add it to ns list to be unload.
-                                return namespaces.stream()
-                                        .filter(namespaceName ->
-                                                policyData.getNamespaces().stream().anyMatch(namespaceName::matches))
-                                        .collect(Collectors.toList());
-                            });
-                }).thenCompose(shouldUnloadNamespaces -> {
-                    if (CollectionUtils.isEmpty(shouldUnloadNamespaces)) {
-                        return CompletableFuture.completedFuture(null);
-                    }
-                    List<CompletableFuture<Void>> futures = shouldUnloadNamespaces.stream()
-                            .map(namespaceName -> adminClient.namespaces().unloadAsync(namespaceName))
-                            .collect(Collectors.toList());
-                    return FutureUtil.waitForAll(futures)
-                            .thenAccept(__ -> {
-                                try {
-                                    // write load info to load manager to make the load happens fast
-                                    pulsar().getLoadManager().get().writeLoadReportOnZookeeper(true);
-                                } catch (Exception e) {
-                                    log.warn("[{}] Failed to writeLoadReportOnZookeeper.", clientAppId(), e);
-                                }
-                            });
-                });
+        // compile regex patterns once
+        List<Pattern> namespacePatterns = policyData.getNamespaces().stream().map(Pattern::compile).toList();
+        // TODO for 4.x, we should include both old and new namespace regex pattern for unload `all_matching` option
+        return adminClient.tenants().getTenantsAsync().thenCompose(tenants -> {
+            List<CompletableFuture<List<String>>> filteredNamespacesForEachTenant = tenants.stream()
+                    .map(tenant -> adminClient.namespaces().getNamespacesAsync(tenant).thenCompose(namespaces -> {
+                        List<CompletableFuture<String>> namespaceNamesInCluster = namespaces.stream()
+                                .filter(namespaceName -> namespacePatterns.stream()
+                                        .anyMatch(pattern -> pattern.matcher(namespaceName).matches()))
+                                .map(namespaceName -> adminClient.namespaces().getPoliciesAsync(namespaceName)
+                                        .thenApply(policies -> policies.replication_clusters.contains(cluster)
+                                                ? namespaceName : null))
+                                .collect(Collectors.toList());
+                        return FutureUtil.waitForAll(namespaceNamesInCluster).thenApply(
+                                __ -> namespaceNamesInCluster.stream()
+                                        .map(CompletableFuture::join)
+                                        .filter(Objects::nonNull)
+                                        .collect(Collectors.toList()));
+                    })).toList();
+            return FutureUtil.waitForAll(filteredNamespacesForEachTenant)
+                    .thenApply(__ -> filteredNamespacesForEachTenant.stream()
+                            .map(CompletableFuture::join)
+                            .flatMap(List::stream)
+                            .collect(Collectors.toList()));
+        }).thenCompose(shouldUnloadNamespaces -> {
+            if (CollectionUtils.isEmpty(shouldUnloadNamespaces)) {
+                return CompletableFuture.completedFuture(null);
+            }
+            // If unload type is 'changed', we need to figure out a further subset of namespaces whose placement might
+            // actually have been changed.
+
+            log.debug("Old policy: {} ; new policy: {}", oldPolicy, policyData);
+            if (oldPolicy != null && NamespaceIsolationPolicyUnloadScope.changed.equals(policyData.getUnloadScope())) {
+                // We also compare that the previous primary broker list is same as current, in case all namespaces need
+                // to be placed again anyway.
+                if (CollectionUtils.isEqualCollection(oldPolicy.getPrimary(), policyData.getPrimary())) {
+                    // list is same, so we continue finding the changed namespaces.
+
+                    // We create a union regex list contains old + new regexes
+                    Set<String> combinedNamespaces = new HashSet<>(oldPolicy.getNamespaces());
+                    combinedNamespaces.addAll(policyData.getNamespaces());
+                    // We create a intersection of the old and new regexes. These won't need to be unloaded
+                    Set<String> commonNamespaces = new HashSet<>(oldPolicy.getNamespaces());
+                    commonNamespaces.retainAll(policyData.getNamespaces());
+
+                    log.debug("combined regexes: {}; common regexes:{}", combinedNamespaces, combinedNamespaces);
+
+                    // Find the changed regexes (new - new ∩ old). TODO for 4.x, make this (new U old - new ∩ old)
+                    combinedNamespaces.removeAll(commonNamespaces);
+
+                    log.debug("changed regexes: {}", commonNamespaces);
+
+                    // Now we further filter the filtered namespaces based on this combinedNamespaces set
+                    shouldUnloadNamespaces = shouldUnloadNamespaces.stream()
+                            .filter(name -> combinedNamespaces.stream()
+                                    .map(Pattern::compile)
+                                    .anyMatch(pattern -> pattern.matcher(name).matches())
+                            ).toList();
+
+                }
+            }
+            // unload type is either null or not in (changed, none), so we proceed to unload all namespaces
+            // TODO - default in 4.x should become `changed`
+            List<CompletableFuture<Void>> futures = shouldUnloadNamespaces.stream()
+                    .map(namespaceName -> adminClient.namespaces().unloadAsync(namespaceName))
+                    .collect(Collectors.toList());
+            return FutureUtil.waitForAll(futures).thenAccept(__ -> {
+                try {
+                    // write load info to load manager to make the load happens fast
+                    pulsar().getLoadManager().get().writeLoadReportOnZookeeper(true);
+                } catch (Exception e) {
+                    log.warn("[{}] Failed to writeLoadReportOnZookeeper.", clientAppId(), e);
+                }
+            });
+        });
     }
 
     @DELETE
@@ -816,7 +878,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The namespace isolation policy name", required = true)
         @PathParam("policyName") String policyName
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterPolicyOperation(cluster, PolicyName.NAMESPACE_ISOLATION, PolicyOperation.WRITE)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> validatePoliciesReadOnlyAccessAsync())
                 .thenCompose(__ -> namespaceIsolationPolicies().getIsolationDataPoliciesAsync(cluster))
@@ -866,7 +928,7 @@ public class ClustersBase extends AdminResource {
         @PathParam("domainName") String domainName,
         @ApiParam(value = "The configuration data of a failure domain", required = true) FailureDomainImpl domain
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.UPDATE_FAILURE_DOMAIN)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> validateBrokerExistsInOtherDomain(cluster, domainName, domain))
                 .thenCompose(__ -> clusterResources().getFailureDomainResources()
@@ -909,7 +971,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The cluster name", required = true)
         @PathParam("cluster") String cluster
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.GET_FAILURE_DOMAIN)
                 .thenCompose(__ -> clusterResources().getFailureDomainResources()
                         .listFailureDomainsAsync(cluster)
                         .thenCompose(domainNames -> {
@@ -965,7 +1027,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The failure domain name", required = true)
         @PathParam("domainName") String domainName
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.GET_FAILURE_DOMAIN)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> clusterResources().getFailureDomainResources()
                         .getFailureDomainAsync(cluster, domainName))
@@ -1000,7 +1062,7 @@ public class ClustersBase extends AdminResource {
         @ApiParam(value = "The failure domain name", required = true)
         @PathParam("domainName") String domainName
     ) {
-        validateSuperUserAccessAsync()
+        validateBothSuperuserAndClusterOperation(cluster, ClusterOperation.DELETE_FAILURE_DOMAIN)
                 .thenCompose(__ -> validateClusterExistAsync(cluster, PRECONDITION_FAILED))
                 .thenCompose(__ -> clusterResources()
                         .getFailureDomainResources().deleteFailureDomainAsync(cluster, domainName))
@@ -1066,5 +1128,126 @@ public class ClustersBase extends AdminResource {
                 });
     }
 
+
+
+    private CompletableFuture<Void> validateBothSuperuserAndClusterOperation(String clusterName,
+                                                                             ClusterOperation operation) {
+        final var superUserAccessValidation = validateSuperUserAccessAsync();
+        final var clusterOperationValidation = validateClusterOperationAsync(clusterName, operation);
+        return FutureUtil.waitForAll(List.of(superUserAccessValidation, clusterOperationValidation))
+                .handle((result, err) -> {
+                    if (!superUserAccessValidation.isCompletedExceptionally()
+                        || !clusterOperationValidation.isCompletedExceptionally()) {
+                        return null;
+                    }
+                    if (log.isDebugEnabled()) {
+                        Throwable superUserValidationException = null;
+                        try {
+                            superUserAccessValidation.join();
+                        } catch (Throwable ex) {
+                            superUserValidationException = FutureUtil.unwrapCompletionException(ex);
+                        }
+                        Throwable clusterOperationValidationException = null;
+                        try {
+                            clusterOperationValidation.join();
+                        } catch (Throwable ex) {
+                            clusterOperationValidationException = FutureUtil.unwrapCompletionException(ex);
+                        }
+                        log.debug("validateBothSuperuserAndClusterOperation failed."
+                                  + " originalPrincipal={} clientAppId={} operation={} cluster={} "
+                                  + "superuserValidationError={} clusterOperationValidationError={}",
+                                originalPrincipal(), clientAppId(), operation.toString(), clusterName,
+                                superUserValidationException, clusterOperationValidationException);
+                    }
+                    throw new RestException(Status.UNAUTHORIZED,
+                            String.format("Unauthorized to validateBothSuperuserAndClusterOperation for"
+                                          + " originalPrincipal [%s] and clientAppId [%s] "
+                                          + "about operation [%s] on cluster [%s]",
+                                    originalPrincipal(), clientAppId(), operation.toString(), clusterName));
+                });
+    }
+
+
+    private CompletableFuture<Void> validateBothSuperuserAndClusterPolicyOperation(String clusterName, PolicyName name,
+                                                                                   PolicyOperation operation) {
+        final var superUserAccessValidation = validateSuperUserAccessAsync();
+        final var clusterOperationValidation = validateClusterPolicyOperationAsync(clusterName, name, operation);
+        return FutureUtil.waitForAll(List.of(superUserAccessValidation, clusterOperationValidation))
+                .handle((result, err) -> {
+                    if (!superUserAccessValidation.isCompletedExceptionally()
+                        || !clusterOperationValidation.isCompletedExceptionally()) {
+                        return null;
+                    }
+                    if (log.isDebugEnabled()) {
+                        Throwable superUserValidationException = null;
+                        try {
+                            superUserAccessValidation.join();
+                        } catch (Throwable ex) {
+                            superUserValidationException = FutureUtil.unwrapCompletionException(ex);
+                        }
+                        Throwable clusterOperationValidationException = null;
+                        try {
+                            clusterOperationValidation.join();
+                        } catch (Throwable ex) {
+                            clusterOperationValidationException = FutureUtil.unwrapCompletionException(ex);
+                        }
+                        log.debug("validateBothSuperuserAndClusterPolicyOperation failed."
+                                  + " originalPrincipal={} clientAppId={} operation={} cluster={} "
+                                  + "superuserValidationError={} clusterOperationValidationError={}",
+                                originalPrincipal(), clientAppId(), operation.toString(), clusterName,
+                                superUserValidationException, clusterOperationValidationException);
+                    }
+                    throw new RestException(Status.UNAUTHORIZED,
+                            String.format("Unauthorized to validateBothSuperuserAndClusterPolicyOperation for"
+                                          + " originalPrincipal [%s] and clientAppId [%s] "
+                                          + "about operation [%s] on cluster [%s]",
+                                    originalPrincipal(), clientAppId(), operation.toString(), clusterName));
+                });
+    }
+
+
+
+
+    private CompletableFuture<Void> validateClusterOperationAsync(String cluster, ClusterOperation operation) {
+        final var pulsar = pulsar();
+        if (pulsar.getBrokerService().isAuthenticationEnabled()
+            && pulsar.getBrokerService().isAuthorizationEnabled()) {
+            return pulsar.getBrokerService().getAuthorizationService()
+                    .allowClusterOperationAsync(cluster, operation, originalPrincipal(),
+                            clientAppId(), clientAuthData())
+                    .thenAccept(isAuthorized -> {
+                        if (!isAuthorized) {
+                            throw new RestException(Status.UNAUTHORIZED,
+                                    String.format("Unauthorized to validateClusterOperation for"
+                                                  + " originalPrincipal [%s] and clientAppId [%s] "
+                                                  + "about operation [%s] on cluster [%s]",
+                                            originalPrincipal(), clientAppId(), operation.toString(), cluster));
+                        }
+                    });
+        }
+        return CompletableFuture.completedFuture(null);
+    }
+
+    private CompletableFuture<Void> validateClusterPolicyOperationAsync(String cluster, PolicyName policyName,
+                                                                        PolicyOperation operation) {
+        final var pulsar = pulsar();
+        if (pulsar.getBrokerService().isAuthenticationEnabled()
+            && pulsar.getBrokerService().isAuthorizationEnabled()) {
+            return pulsar.getBrokerService().getAuthorizationService()
+                    .allowClusterPolicyOperationAsync(cluster, policyName, operation, originalPrincipal(),
+                            clientAppId(), clientAuthData())
+                    .thenAccept(isAuthorized -> {
+                        if (!isAuthorized) {
+                            throw new RestException(Status.UNAUTHORIZED,
+                                    String.format("Unauthorized to validateClusterPolicyOperation for"
+                                                  + " originalPrincipal [%s] and clientAppId [%s] "
+                                                  + "about operation [%s] on cluster [%s]",
+                                            originalPrincipal(), clientAppId(), operation.toString(), cluster));
+                        }
+                    });
+        }
+        return CompletableFuture.completedFuture(null);
+    }
+
     private static final Logger log = LoggerFactory.getLogger(ClustersBase.class);
 }
