@@ -25,6 +25,7 @@ import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
 import org.apache.pulsar.broker.authentication.AuthenticationDataSource;
 import org.apache.pulsar.broker.loadbalance.extensions.data.BrokerLookupData;
+import org.apache.pulsar.common.api.proto.FeatureFlags;
 
 public interface TransportCnx {
 
@@ -82,9 +83,9 @@ public interface TransportCnx {
      * by actively sending a Ping message to the client.
      *
      * @return a completable future where the result is true if the connection is alive, false otherwise. The result
-     * is null if the connection liveness check is disabled.
+     * is empty if the connection liveness check is disabled.
      */
-    CompletableFuture<Boolean> checkConnectionLiveness();
+    CompletableFuture<Optional<Boolean>> checkConnectionLiveness();
 
     /**
      * Increments the counter that controls the throttling of the connection by pausing reads.
@@ -103,4 +104,11 @@ public interface TransportCnx {
      * previously called {@link #incrementThrottleCount()}.
      */
     void decrementThrottleCount();
+
+    FeatureFlags getFeatures();
+
+    default boolean isClientSupportsReplDedupByLidAndEid() {
+        return getFeatures() != null && getFeatures().hasSupportsReplDedupByLidAndEid()
+                && getFeatures().isSupportsReplDedupByLidAndEid();
+    }
 }
