@@ -85,11 +85,26 @@ public class PersistentStickyKeyDispatcherMultipleConsumers extends PersistentDi
     public boolean sortRecentlyJoinedConsumersIfNeeded = true;
 
     PersistentStickyKeyDispatcherMultipleConsumers(PersistentTopic topic, ManagedCursor cursor,
-            Subscription subscription, ServiceConfiguration conf, KeySharedMeta ksm) {
+                                                          Subscription subscription, ServiceConfiguration conf,
+                                                          KeySharedMeta ksm) {
+        this(topic, cursor, subscription, conf, ksm, null);
+    }
+
+    /**
+     * @param recentlyJoinedConsumers This parameter is only used for testing.
+     */
+    @VisibleForTesting
+    PersistentStickyKeyDispatcherMultipleConsumers(PersistentTopic topic, ManagedCursor cursor,
+            Subscription subscription, ServiceConfiguration conf, KeySharedMeta ksm,
+                                                   LinkedHashMap<Consumer, PositionImpl> recentlyJoinedConsumers) {
         super(topic, cursor, subscription, ksm.isAllowOutOfOrderDelivery());
 
         this.allowOutOfOrderDelivery = ksm.isAllowOutOfOrderDelivery();
-        this.recentlyJoinedConsumers = allowOutOfOrderDelivery ? null : new LinkedHashMap<>();
+        if (recentlyJoinedConsumers == null) {
+            this.recentlyJoinedConsumers = allowOutOfOrderDelivery ? null : new LinkedHashMap<>();
+        } else {
+            this.recentlyJoinedConsumers = recentlyJoinedConsumers;
+        }
         this.keySharedMode = ksm.getKeySharedMode();
         switch (this.keySharedMode) {
         case AUTO_SPLIT:
@@ -154,6 +169,10 @@ public class PersistentStickyKeyDispatcherMultipleConsumers extends PersistentDi
         });
     }
 
+    /**
+     * Sort items in the collection "recentlyJoinedConsumers" if needed.
+     * Since we check the order of queue after each consumer joined, we can only check the last two items.
+     */
     private void sortRecentlyJoinedConsumersIfNeeded() {
         if (!sortRecentlyJoinedConsumersIfNeeded) {
             return;
@@ -161,20 +180,21 @@ public class PersistentStickyKeyDispatcherMultipleConsumers extends PersistentDi
         if (recentlyJoinedConsumers.size() == 1) {
             return;
         }
-        // Since we check the order of queue after each consumer joined, we can only check the last two items.
         boolean sortNeeded = false;
-        PositionImpl posPre = null;
-        PositionImpl posAfter = null;
+        PositionImpl secondLatest = null;
+        PositionImpl latest = null;
         for (Map.Entry<Consumer, PositionImpl> entry : recentlyJoinedConsumers.entrySet()) {
-            if (posPre == null) {
-                posPre = entry.getValue();
+            if (secondLatest == null) {
+                secondLatest = entry.getValue();
+            } else if (latest == null) {
+                latest = entry.getValue();
             } else {
-                posPre = posAfter;
-                posAfter = entry.getValue();
+                secondLatest = latest;
+                latest = entry.getValue();
             }
         }
-        if (posPre != null && posAfter != null) {
-            if (posPre.compareTo(posAfter) > 0) {
+        if (secondLatest != null && latest != null) {
+            if (secondLatest.compareTo(latest) > 0) {
                 sortNeeded = true;
             }
         }
