@@ -18,7 +18,6 @@
  */
 package org.apache.pulsar.broker.stats;
 
-import io.opentelemetry.api.common.Attributes;
 import io.opentelemetry.api.metrics.BatchCallback;
 import io.opentelemetry.api.metrics.ObservableDoubleMeasurement;
 import io.opentelemetry.api.metrics.ObservableLongMeasurement;
@@ -31,12 +30,10 @@ import org.apache.pulsar.broker.service.Dispatcher;
 import org.apache.pulsar.broker.service.Subscription;
 import org.apache.pulsar.broker.service.Topic;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
-import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.BacklogQuota;
 import org.apache.pulsar.common.stats.MetricsUtil;
 import org.apache.pulsar.compaction.CompactedTopicContext;
 import org.apache.pulsar.compaction.Compactor;
-import org.apache.pulsar.opentelemetry.OpenTelemetryAttributes;
 
 public class OpenTelemetryTopicStats implements AutoCloseable {
 
@@ -383,16 +380,8 @@ public class OpenTelemetryTopicStats implements AutoCloseable {
     }
 
     private void recordMetricsForTopic(Topic topic) {
-        var topicName = TopicName.get(topic.getName());
-        var builder = Attributes.builder()
-                .put(OpenTelemetryAttributes.PULSAR_DOMAIN, topicName.getDomain().toString())
-                .put(OpenTelemetryAttributes.PULSAR_TENANT, topicName.getTenant())
-                .put(OpenTelemetryAttributes.PULSAR_NAMESPACE, topicName.getNamespace())
-                .put(OpenTelemetryAttributes.PULSAR_TOPIC, topicName.getPartitionedTopicName());
-        if (topicName.isPartitioned()) {
-            builder.put(OpenTelemetryAttributes.PULSAR_PARTITION_INDEX, topicName.getPartitionIndex());
-        }
-        var attributes = builder.build();
+        var topicAttributes = topic.getTopicAttributes();
+        var attributes = topicAttributes.getCommonAttributes();
 
         if (topic instanceof AbstractTopic abstractTopic) {
             subscriptionCounter.record(abstractTopic.getSubscriptions().size(), attributes);
@@ -410,6 +399,7 @@ public class OpenTelemetryTopicStats implements AutoCloseable {
         }
 
         if (topic instanceof PersistentTopic persistentTopic) {
+            var persistentTopicAttributes = persistentTopic.getTopicAttributes();
             var managedLedger = persistentTopic.getManagedLedger();
             var managedLedgerStats = persistentTopic.getManagedLedger().getStats();
             storageCounter.record(managedLedgerStats.getStoredMessagesSize(), attributes);
@@ -428,29 +418,17 @@ public class OpenTelemetryTopicStats implements AutoCloseable {
             backlogQuotaAge.record(topic.getBestEffortOldestUnacknowledgedMessageAgeSeconds(), attributes);
             var backlogQuotaMetrics = persistentTopic.getPersistentTopicMetrics().getBacklogQuotaMetrics();
             backlogEvictionCounter.record(backlogQuotaMetrics.getSizeBasedBacklogQuotaExceededEvictionCount(),
-                    Attributes.builder()
-                            .putAll(attributes)
-                            .put(OpenTelemetryAttributes.PULSAR_BACKLOG_QUOTA_TYPE, "size")
-                            .build());
+                    persistentTopicAttributes.getSizeBasedQuotaAttributes());
             backlogEvictionCounter.record(backlogQuotaMetrics.getTimeBasedBacklogQuotaExceededEvictionCount(),
-                    Attributes.builder()
-                            .putAll(attributes)
-                            .put(OpenTelemetryAttributes.PULSAR_BACKLOG_QUOTA_TYPE, "time")
-                            .build());
+                    persistentTopicAttributes.getTimeBasedQuotaAttributes());
 
             var txnBuffer = persistentTopic.getTransactionBuffer();
-            transactionCounter.record(txnBuffer.getOngoingTxnCount(), Attributes.builder()
-                    .putAll(attributes)
-                    .put(OpenTelemetryAttributes.PULSAR_TRANSACTION_STATUS, "active")
-                    .build());
-            transactionCounter.record(txnBuffer.getCommittedTxnCount(), Attributes.builder()
-                    .putAll(attributes)
-                    .put(OpenTelemetryAttributes.PULSAR_TRANSACTION_STATUS, "committed")
-                    .build());
-            transactionCounter.record(txnBuffer.getAbortedTxnCount(), Attributes.builder()
-                    .putAll(attributes)
-                    .put(OpenTelemetryAttributes.PULSAR_TRANSACTION_STATUS, "aborted")
-                    .build());
+            transactionCounter.record(txnBuffer.getOngoingTxnCount(),
+                    persistentTopicAttributes.getTransactionActiveAttributes());
+            transactionCounter.record(txnBuffer.getCommittedTxnCount(),
+                    persistentTopicAttributes.getTransactionCommittedAttributes());
+            transactionCounter.record(txnBuffer.getAbortedTxnCount(),
+                    persistentTopicAttributes.getTransactionAbortedAttributes());
 
             Optional.ofNullable(pulsar.getNullableCompactor())
                     .map(Compactor::getStats)
@@ -458,15 +436,9 @@ public class OpenTelemetryTopicStats implements AutoCloseable {
                     .ifPresent(compactionRecord -> {
                         compactionRemovedCounter.record(compactionRecord.getCompactionRemovedEventCount(), attributes);
                         compactionOperationCounter.record(compactionRecord.getCompactionSucceedCount(),
-                                Attributes.builder()
-                                        .putAll(attributes)
-                                        .put(OpenTelemetryAttributes.PULSAR_COMPACTION_STATUS, "success")
-                                        .build());
+                                persistentTopicAttributes.getCompactionSuccessAttributes());
                         compactionOperationCounter.record(compactionRecord.getCompactionFailedCount(),
-                                Attributes.builder()
-                                        .putAll(attributes)
-                                        .put(OpenTelemetryAttributes.PULSAR_COMPACTION_STATUS, "failure")
-                                        .build());
+                                persistentTopicAttributes.getCompactionFailureAttributes());
                         compactionDurationSeconds.record(MetricsUtil.convertToSeconds(
                             compactionRecord.getCompactionDurationTimeInMills(), TimeUnit.MILLISECONDS), attributes);
                         compactionBytesInCounter.record(compactionRecord.getCompactionReadBytes(), attributes);
