@@ -19,36 +19,28 @@
 package org.apache.pulsar.compaction;
 
 import static com.google.common.base.Preconditions.checkArgument;
-import static org.apache.pulsar.common.protocol.Commands.DEFAULT_CONSUMER_EPOCH;
+import static org.apache.bookkeeper.mledger.util.ManagedLedgerUtils.readEntries;
+import static org.apache.bookkeeper.mledger.util.ManagedLedgerUtils.readEntriesWithSkipOrWait;
 import com.google.common.annotations.Beta;
-import java.util.Collections;
+import java.util.List;
 import java.util.Objects;
 import java.util.concurrent.CompletableFuture;
-import org.apache.bookkeeper.mledger.AsyncCallbacks;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedCursor;
-import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
 import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.apache.commons.collections4.CollectionUtils;
-import org.apache.pulsar.broker.service.Consumer;
-import org.apache.pulsar.broker.service.persistent.PersistentDispatcherSingleActiveConsumer;
-import org.apache.pulsar.common.util.FutureUtil;
-import org.jspecify.annotations.Nullable;
 
 public class CompactedTopicUtils {
 
     @Beta
-    public static void asyncReadCompactedEntries(TopicCompactionService topicCompactionService,
-                                                 ManagedCursor cursor, int maxEntries,
-                                                 long bytesToRead, PositionImpl maxReadPosition,
-                                                 boolean readFromEarliest, AsyncCallbacks.ReadEntriesCallback callback,
-                                                 boolean wait, @Nullable Consumer consumer) {
+    public static CompletableFuture<List<Entry>> asyncReadCompactedEntries(
+            TopicCompactionService topicCompactionService, ManagedCursor cursor, int maxEntries,
+            long bytesToRead, PositionImpl maxReadPosition, boolean readFromEarliest, boolean wait) {
         Objects.requireNonNull(topicCompactionService);
         Objects.requireNonNull(cursor);
         checkArgument(maxEntries > 0);
-        Objects.requireNonNull(callback);
 
         final PositionImpl readPosition;
         if (readFromEarliest) {
@@ -57,37 +49,31 @@ public class CompactedTopicUtils {
             readPosition = (PositionImpl) cursor.getReadPosition();
         }
 
-        // TODO: redeliver epoch link https://github.com/apache/pulsar/issues/13690
-        PersistentDispatcherSingleActiveConsumer.ReadEntriesCtx readEntriesCtx =
-                PersistentDispatcherSingleActiveConsumer.ReadEntriesCtx.create(consumer, DEFAULT_CONSUMER_EPOCH);
-
         CompletableFuture<Position> lastCompactedPositionFuture = topicCompactionService.getLastCompactedPosition();
 
-        lastCompactedPositionFuture.thenCompose(lastCompactedPosition -> {
+        return lastCompactedPositionFuture.thenCompose(lastCompactedPosition -> {
             if (lastCompactedPosition == null
                     || readPosition.compareTo(
                     lastCompactedPosition.getLedgerId(), lastCompactedPosition.getEntryId()) > 0) {
                 if (wait) {
-                    cursor.asyncReadEntriesOrWait(maxEntries, bytesToRead, callback, readEntriesCtx, maxReadPosition);
+                    return readEntriesWithSkipOrWait(cursor, maxEntries, bytesToRead, maxReadPosition, null);
                 } else {
-                    cursor.asyncReadEntries(maxEntries, bytesToRead, callback, readEntriesCtx, maxReadPosition);
+                    return readEntries(cursor, maxEntries, bytesToRead, maxReadPosition);
                 }
-                return CompletableFuture.completedFuture(null);
             }
 
             ManagedCursorImpl managedCursor = (ManagedCursorImpl) cursor;
             int numberOfEntriesToRead = managedCursor.applyMaxSizeCap(maxEntries, bytesToRead);
 
             return topicCompactionService.readCompactedEntries(readPosition, numberOfEntriesToRead)
-                    .thenAccept(entries -> {
+                    .thenApply(entries -> {
                         if (CollectionUtils.isEmpty(entries)) {
                             Position seekToPosition = lastCompactedPosition.getNext();
                             if (readPosition.compareTo(seekToPosition.getLedgerId(), seekToPosition.getEntryId()) > 0) {
                                 seekToPosition = readPosition;
                             }
                             cursor.seek(seekToPosition);
-                            callback.readEntriesComplete(Collections.emptyList(), readEntriesCtx);
-                            return;
+                            return entries;
                         }
 
                         long entriesSize = 0;
@@ -98,12 +84,8 @@ public class CompactedTopicUtils {
 
                         Entry lastEntry = entries.get(entries.size() - 1);
                         cursor.seek(lastEntry.getPosition().getNext(), true);
-                        callback.readEntriesComplete(entries, readEntriesCtx);
+                        return entries;
                     });
-        }).exceptionally((exception) -> {
-            exception = FutureUtil.unwrapCompletionException(exception);
-            callback.readEntriesFailed(ManagedLedgerException.getManagedLedgerException(exception), readEntriesCtx);
-            return null;
         });
     }
 }
