@@ -45,6 +45,8 @@ public class CompactionRecord {
     private final LongAdder compactionSucceedCount = new LongAdder();
     private final LongAdder compactionFailedCount = new LongAdder();
     private final LongAdder compactionDurationTimeInMills = new LongAdder();
+    private final LongAdder compactionReadBytes = new LongAdder();
+    private final LongAdder compactionWriteBytes = new LongAdder();
     public final StatsBuckets writeLatencyStats = new StatsBuckets(WRITE_LATENCY_BUCKETS_USEC);
     public final Rate writeRate = new Rate();
     public final Rate readRate = new Rate();
@@ -83,10 +85,12 @@ public class CompactionRecord {
 
     public void addCompactionReadOp(long readableBytes) {
         readRate.recordEvent(readableBytes);
+        compactionReadBytes.add(readableBytes);
     }
 
     public void addCompactionWriteOp(long writeableBytes) {
         writeRate.recordEvent(writeableBytes);
+        compactionWriteBytes.add(writeableBytes);
     }
 
     public void addCompactionLatencyOp(long latency, TimeUnit unit) {
@@ -115,7 +119,6 @@ public class CompactionRecord {
     }
 
     public StatsBuckets getCompactionLatencyStats() {
-        writeLatencyStats.refresh();
         return writeLatencyStats;
     }
 
@@ -124,8 +127,16 @@ public class CompactionRecord {
         return readRate.getValueRate();
     }
 
+    public long getCompactionReadBytes() {
+        return compactionReadBytes.sum();
+    }
+
     public double getCompactionWriteThroughput() {
         writeRate.calculateRate();
         return writeRate.getValueRate();
     }
+
+    public long getCompactionWriteBytes() {
+        return compactionWriteBytes.sum();
+    }
 }
