@@ -36,11 +36,12 @@ import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.net.CachedDNSToSwitchMapping;
 import org.apache.bookkeeper.stats.StatsLogger;
-import org.apache.commons.configuration.ConfigurationException;
+import org.apache.commons.configuration2.ex.ConfigurationException;
 import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.pulsar.bookie.rackawareness.BookieRackAffinityMapping;
 import org.apache.pulsar.metadata.api.MetadataStore;
 import org.apache.pulsar.metadata.api.extended.MetadataStoreExtended;
+import org.apache.pulsar.zookeeper.ZkIsolatedBookieEnsemblePlacementPolicy;
 import org.testng.annotations.Test;
 
 /**
@@ -152,6 +153,24 @@ public class BookKeeperClientFactoryImplTest {
         assertEquals(20, bkConf.getMinNumRacksPerWriteQuorum());
     }
 
+    @Test
+    public void testSetEnsemblePlacementPolicys() {
+        ClientConfiguration bkConf = new ClientConfiguration();
+        ServiceConfiguration conf = new ServiceConfiguration();
+        conf.setBookkeeperClientMinNumRacksPerWriteQuorum(3);
+        conf.setBookkeeperClientEnforceMinNumRacksPerWriteQuorum(true);
+
+        MetadataStore store = mock(MetadataStore.class);
+
+        BookKeeperClientFactoryImpl.setEnsemblePlacementPolicy(
+                bkConf,
+                conf,
+                store,
+                ZkIsolatedBookieEnsemblePlacementPolicy.class);
+        assertEquals(bkConf.getMinNumRacksPerWriteQuorum(), 3);
+        assertTrue(bkConf.getEnforceMinNumRacksPerWriteQuorum());
+    }
+
     @Test
     public void testSetDiskWeightBasedPlacementEnabled() {
         BookKeeperClientFactoryImpl factory = new BookKeeperClientFactoryImpl();
