@@ -45,7 +45,7 @@ import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 @Slf4j
-@Test(groups = "broker-replication")
+@Test(groups = "broker")
 public class ReplicationTopicGcTest extends OneWayReplicatorTestBase {
 
     @Override
@@ -118,10 +118,10 @@ public class ReplicationTopicGcTest extends OneWayReplicatorTestBase {
             assertFalse(pulsar2.getPulsarResources().getNamespaceResources()
                     .getPartitionedTopicResources().partitionedTopicExists(TopicName.get(topicName)));
             // topic policies.
-            assertTrue(pulsar1.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName))
-                    .get().isEmpty());
-            assertTrue(pulsar2.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName))
-                    .get().isEmpty());
+            assertTrue(pulsar1.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName),
+                    false).get().isEmpty());
+            assertTrue(pulsar2.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName),
+                    false).get().isEmpty());
             // schema.
             assertTrue(CollectionUtils.isEmpty(pulsar1.getSchemaStorage().getAll(schemaId).get()));
             assertTrue(CollectionUtils.isEmpty(pulsar2.getSchemaStorage().getAll(schemaId).get()));
@@ -168,8 +168,8 @@ public class ReplicationTopicGcTest extends OneWayReplicatorTestBase {
             assertFalse(pulsar1.getPulsarResources().getNamespaceResources()
                     .getPartitionedTopicResources().partitionedTopicExists(TopicName.get(topicName)));
             // topic policies.
-            assertTrue(pulsar1.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName))
-                    .get().isEmpty());
+            assertTrue(pulsar1.getTopicPoliciesService().getTopicPoliciesAsync(TopicName.get(topicName),
+                    false).get().isEmpty());
             // schema.
             assertTrue(CollectionUtils.isEmpty(pulsar1.getSchemaStorage().getAll(schemaId).get()));
         });
