@@ -30,7 +30,7 @@ import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
-@Test(groups = "broker-replication")
+@Test(groups = "broker")
 public class ReplicatorTlsTest extends ReplicatorTestBase {
 
     @Override
