@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,6 +19,7 @@
 package org.apache.pulsar.broker.service.plugin;
 
 
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
@@ -35,25 +36,28 @@ public class EntryFilterTest implements EntryFilter {
             return FilterResult.ACCEPT;
         }
         Consumer consumer = context.getConsumer();
-        Map<String, String> metadata = consumer.getMetadata();
+        Map<String, String> metadata = consumer != null ? consumer.getMetadata() : Collections.emptyMap();
         log.info("filterEntry for {}", metadata);
         String matchValueAccept = metadata.getOrDefault("matchValueAccept", "ACCEPT");
         String matchValueReject = metadata.getOrDefault("matchValueReject", "REJECT");
         String matchValueReschedule = metadata.getOrDefault("matchValueReschedule", "RESCHEDULE");
         List<KeyValue> list = context.getMsgMetadata().getPropertiesList();
+        String debug =
+                list.stream().filter(kv -> "debug".equalsIgnoreCase(kv.getKey())).findFirst().map(KeyValue::getValue)
+                        .orElse("-");
         // filter by string
         for (KeyValue keyValue : list) {
             if (matchValueAccept.equalsIgnoreCase(keyValue.getKey())) {
-                log.info("metadata {} key {} outcome ACCEPT", metadata, keyValue.getKey());
+                log.info("metadata {} key {} debug '{}' outcome ACCEPT", metadata, keyValue.getKey(), debug);
                 return FilterResult.ACCEPT;
             } else if (matchValueReject.equalsIgnoreCase(keyValue.getKey())){
-                log.info("metadata {} key {} outcome REJECT", metadata, keyValue.getKey());
+                log.info("metadata {} key {} debug '{}' outcome REJECT", metadata, keyValue.getKey(), debug);
                 return FilterResult.REJECT;
             } else if (matchValueReschedule.equalsIgnoreCase(keyValue.getKey())){
-                log.info("metadata {} key {} outcome RESCHEDULE", metadata, keyValue.getKey());
+                log.info("metadata {} key {} debug '{}' outcome RESCHEDULE", metadata, keyValue.getKey(), debug);
                 return FilterResult.RESCHEDULE;
             } else {
-                log.info("metadata {} key {} outcome ??", metadata, keyValue.getKey());
+                log.info("metadata {} key {} debug '{}' outcome ??", metadata, keyValue.getKey(), debug);
             }
         }
         return null;
