@@ -73,7 +73,7 @@ import org.apache.zookeeper.MockZooKeeper;
 import org.apache.zookeeper.MockZooKeeperSession;
 import org.apache.zookeeper.ZooDefs;
 import org.apache.zookeeper.ZooKeeper;
-import org.jetbrains.annotations.NotNull;
+import org.jspecify.annotations.NonNull;
 import org.mockito.Mockito;
 import org.mockito.internal.util.MockUtil;
 
@@ -231,6 +231,18 @@ public class PulsarTestContext implements AutoCloseable {
                 getPulsarService());
     }
 
+    private enum WithMockZooKeeperOrTestZKServer {
+        MOCKZOOKEEPER, MOCKZOOKEEPER_SEPARATE_GLOBAL, TEST_ZK_SERVER, TEST_ZK_SERVER_SEPARATE_GLOBAL;
+
+        boolean isMockZooKeeper() {
+            return this == MOCKZOOKEEPER || this == MOCKZOOKEEPER_SEPARATE_GLOBAL;
+        }
+
+        boolean isTestZKServer() {
+            return this == TEST_ZK_SERVER || this == TEST_ZK_SERVER_SEPARATE_GLOBAL;
+        }
+    }
+
     /**
      * A builder for a PulsarTestContext.
      *
@@ -247,6 +259,7 @@ public class PulsarTestContext implements AutoCloseable {
         protected boolean configOverrideCalled = false;
         protected Function<BrokerService, BrokerService> brokerServiceCustomizer = Function.identity();
         protected PulsarTestContext otherContextToClose;
+        protected WithMockZooKeeperOrTestZKServer withMockZooKeeperOrTestZKServer;
 
         /**
          * Initialize the ServiceConfiguration with default values.
@@ -403,11 +416,13 @@ public class PulsarTestContext implements AutoCloseable {
         public Builder reuseMockBookkeeperAndMetadataStores(PulsarTestContext otherContext) {
             bookKeeperClient(otherContext.getBookKeeperClient());
             if (otherContext.getMockZooKeeper() != null) {
+                withMockZooKeeperOrTestZKServer = null;
                 mockZooKeeper(otherContext.getMockZooKeeper());
                 if (otherContext.getMockZooKeeperGlobal() != null) {
                     mockZooKeeperGlobal(otherContext.getMockZooKeeperGlobal());
                 }
             } else if (otherContext.getTestZKServer() != null) {
+                withMockZooKeeperOrTestZKServer = null;
                 testZKServer(otherContext.getTestZKServer());
                 if (otherContext.getTestZKServerGlobal() != null) {
                     testZKServerGlobal(otherContext.getTestZKServerGlobal());
@@ -467,31 +482,14 @@ public class PulsarTestContext implements AutoCloseable {
          * @return the builder
          */
         public Builder withMockZookeeper(boolean useSeparateGlobalZk) {
-            try {
-                mockZooKeeper(createMockZooKeeper());
-                if (useSeparateGlobalZk) {
-                    mockZooKeeperGlobal(createMockZooKeeper());
-                }
-            } catch (Exception e) {
-                throw new RuntimeException(e);
+            if (useSeparateGlobalZk) {
+                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER_SEPARATE_GLOBAL;
+            } else {
+                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER;
             }
             return this;
         }
 
-        private MockZooKeeper createMockZooKeeper() throws Exception {
-            MockZooKeeper zk = MockZooKeeper.newInstance();
-            initializeZookeeper(zk);
-            registerCloseable(zk::shutdown);
-            return zk;
-        }
-
-        private static void initializeZookeeper(ZooKeeper zk) throws KeeperException, InterruptedException {
-            ZkUtils.createFullPathOptimistic(zk, "/ledgers/available/192.168.1.1:" + 5000,
-                    "".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
-
-            zk.create("/ledgers/LAYOUT", "1\nflat:1".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE,
-                    CreateMode.PERSISTENT);
-        }
 
         /**
          * Configure this PulsarTestContext to use a test ZooKeeper instance which is
@@ -510,27 +508,14 @@ public class PulsarTestContext implements AutoCloseable {
          * @return the builder
          */
         public Builder withTestZookeeper(boolean useSeparateGlobalZk) {
-            try {
-                testZKServer(createTestZookeeper());
-                if (useSeparateGlobalZk) {
-                    testZKServerGlobal(createTestZookeeper());
-                }
-            } catch (Exception e) {
-                throw new RuntimeException(e);
+            if (useSeparateGlobalZk) {
+                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER_SEPARATE_GLOBAL;
+            } else {
+                withMockZooKeeperOrTestZKServer = WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER;
             }
             return this;
         }
 
-        private TestZKServer createTestZookeeper() throws Exception {
-            TestZKServer testZKServer = new TestZKServer();
-            try (ZooKeeper zkc = new ZooKeeper(testZKServer.getConnectionString(), 5000, event -> {
-            })) {
-                initializeZookeeper(zkc);
-            }
-            registerCloseable(testZKServer);
-            return testZKServer;
-        }
-
         /**
          * Applicable only when PulsarTestContext is not startable. This will configure mocks
          * for PulsarTestResources and related classes.
@@ -618,6 +603,7 @@ public class PulsarTestContext implements AutoCloseable {
             if (configOverrideCustomizer != null) {
                 configOverrideCustomizer.accept(super.config);
             }
+            createWithMockZooKeeperOrTestZKServerInstances();
             initializeCommonPulsarServices(spyConfig);
             initializePulsarServices(spyConfig, this);
             if (pulsarServiceCustomizer != null) {
@@ -639,6 +625,73 @@ public class PulsarTestContext implements AutoCloseable {
             return super.build();
         }
 
+        void createWithMockZooKeeperOrTestZKServerInstances() {
+            if (withMockZooKeeperOrTestZKServer == null) {
+                return;
+            }
+            int sessionTimeout = (int) super.config.getMetadataStoreSessionTimeoutMillis();
+            try {
+                if (withMockZooKeeperOrTestZKServer.isMockZooKeeper()) {
+                    if (super.mockZooKeeper == null) {
+                        mockZooKeeper(createMockZooKeeper(sessionTimeout));
+                    } else {
+                        log.warn("Skipping creating mockZooKeeper, already set");
+                    }
+                    if (withMockZooKeeperOrTestZKServer
+                            == WithMockZooKeeperOrTestZKServer.MOCKZOOKEEPER_SEPARATE_GLOBAL) {
+                        if (super.mockZooKeeperGlobal == null) {
+                            mockZooKeeperGlobal(createMockZooKeeper(sessionTimeout));
+                        } else {
+                            log.warn("Skipping creating mockZooKeeperGlobal, already set");
+                        }
+                    }
+                } else if (withMockZooKeeperOrTestZKServer.isTestZKServer()) {
+                    if (super.testZKServer == null) {
+                        testZKServer(createTestZookeeper(sessionTimeout));
+                    } else {
+                        log.warn("Skipping creating testZKServer, already set");
+                    }
+                    if (withMockZooKeeperOrTestZKServer
+                            == WithMockZooKeeperOrTestZKServer.TEST_ZK_SERVER_SEPARATE_GLOBAL) {
+                        if (super.testZKServerGlobal == null) {
+                            testZKServerGlobal(createTestZookeeper(sessionTimeout));
+                        } else {
+                            log.warn("Skipping creating testZKServerGlobal, already set");
+                        }
+                    }
+                }
+            } catch (Exception e) {
+                throw new RuntimeException(e);
+            }
+        }
+
+        private MockZooKeeper createMockZooKeeper(int sessionTimeout) throws Exception {
+            MockZooKeeper zk = MockZooKeeper.newInstance();
+            zk.setSessionTimeout(sessionTimeout);
+            initializeZookeeper(zk);
+            registerCloseable(zk::shutdown);
+            return zk;
+        }
+
+        // this might not be required at all, but it's kept here as an example
+        private static void initializeZookeeper(ZooKeeper zk) throws KeeperException, InterruptedException {
+            ZkUtils.createFullPathOptimistic(zk, "/ledgers/available/192.168.1.1:" + 5000,
+                    "".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+
+            zk.create("/ledgers/LAYOUT", "1\nflat:1".getBytes(StandardCharsets.UTF_8), ZooDefs.Ids.OPEN_ACL_UNSAFE,
+                    CreateMode.PERSISTENT);
+        }
+
+        private TestZKServer createTestZookeeper(int sessionTimeout) throws Exception {
+            TestZKServer testZKServer = new TestZKServer();
+            try (ZooKeeper zkc = new ZooKeeper(testZKServer.getConnectionString(), sessionTimeout, event -> {
+            })) {
+                initializeZookeeper(zkc);
+            }
+            registerCloseable(testZKServer);
+            return testZKServer;
+        }
+
         protected void handlePreallocatePorts(ServiceConfiguration config) {
             if (super.preallocatePorts) {
                 config.getBrokerServicePort().ifPresent(portNumber -> {
@@ -700,28 +753,30 @@ public class PulsarTestContext implements AutoCloseable {
             if (super.localMetadataStore == null || super.configurationMetadataStore == null) {
                 if (super.mockZooKeeper != null) {
                     MetadataStoreExtended mockZookeeperMetadataStore =
-                            createMockZookeeperMetadataStore(super.mockZooKeeper, MetadataStoreConfig.METADATA_STORE);
+                            createMockZookeeperMetadataStore(super.mockZooKeeper, super.config,
+                                    MetadataStoreConfig.METADATA_STORE);
                     if (super.localMetadataStore == null) {
                         localMetadataStore(mockZookeeperMetadataStore);
                     }
                     if (super.configurationMetadataStore == null) {
                         if (super.mockZooKeeperGlobal != null) {
                             configurationMetadataStore(createMockZookeeperMetadataStore(super.mockZooKeeperGlobal,
-                                    MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
+                                    super.config, MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
                         } else {
                             configurationMetadataStore(mockZookeeperMetadataStore);
                         }
                     }
                 } else if (super.testZKServer != null) {
                     MetadataStoreExtended testZookeeperMetadataStore =
-                            createTestZookeeperMetadataStore(super.testZKServer, MetadataStoreConfig.METADATA_STORE);
+                            createTestZookeeperMetadataStore(super.testZKServer, super.config,
+                                    MetadataStoreConfig.METADATA_STORE);
                     if (super.localMetadataStore == null) {
                         localMetadataStore(testZookeeperMetadataStore);
                     }
                     if (super.configurationMetadataStore == null) {
                         if (super.testZKServerGlobal != null) {
                             configurationMetadataStore(createTestZookeeperMetadataStore(super.testZKServerGlobal,
-                                    MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
+                                    super.config, MetadataStoreConfig.CONFIGURATION_METADATA_STORE));
                         } else {
                             configurationMetadataStore(testZookeeperMetadataStore);
                         }
@@ -751,16 +806,30 @@ public class PulsarTestContext implements AutoCloseable {
             }
         }
 
+        private MetadataStoreConfig createMetadataStoreConfig(ServiceConfiguration config, String metadataStoreName) {
+            return MetadataStoreConfig.builder()
+                    .sessionTimeoutMillis((int) config.getMetadataStoreSessionTimeoutMillis())
+                    .allowReadOnlyOperations(config.isMetadataStoreAllowReadOnlyOperations())
+                    .batchingEnabled(config.isMetadataStoreBatchingEnabled())
+                    .batchingMaxDelayMillis(config.getMetadataStoreBatchingMaxDelayMillis())
+                    .batchingMaxOperations(config.getMetadataStoreBatchingMaxOperations())
+                    .batchingMaxSizeKb(config.getMetadataStoreBatchingMaxSizeKb())
+                    .metadataStoreName(metadataStoreName)
+                    .build();
+        }
+
         private MetadataStoreExtended createMockZookeeperMetadataStore(MockZooKeeper mockZooKeeper,
+                                                                       ServiceConfiguration config,
                                                                        String metadataStoreName) {
             // provide a unique session id for each instance
             MockZooKeeperSession mockZooKeeperSession = MockZooKeeperSession.newInstance(mockZooKeeper, false);
+            mockZooKeeperSession.setSessionTimeout((int) config.getMetadataStoreSessionTimeoutMillis());
             registerCloseable(() -> {
                 mockZooKeeperSession.close();
                 resetSpyOrMock(mockZooKeeperSession);
             });
-            ZKMetadataStore zkMetadataStore = new ZKMetadataStore(mockZooKeeperSession,
-                    MetadataStoreConfig.builder().metadataStoreName(metadataStoreName).build());
+            ZKMetadataStore zkMetadataStore =
+                    new ZKMetadataStore(mockZooKeeperSession, createMetadataStoreConfig(config, metadataStoreName));
             registerCloseable(() -> {
                 zkMetadataStore.close();
                 resetSpyOrMock(zkMetadataStore);
@@ -772,9 +841,10 @@ public class PulsarTestContext implements AutoCloseable {
 
         @SneakyThrows
         private MetadataStoreExtended createTestZookeeperMetadataStore(TestZKServer zkServer,
+                                                                       ServiceConfiguration config,
                                                                        String metadataStoreName) {
             MetadataStoreExtended store = MetadataStoreExtended.create("zk:" + zkServer.getConnectionString(),
-                    MetadataStoreConfig.builder().metadataStoreName(metadataStoreName).build());
+                    createMetadataStoreConfig(config, metadataStoreName));
             registerCloseable(store);
             MetadataStoreExtended nonClosingProxy =
                     NonClosingProxyHandler.createNonClosingProxy(store, MetadataStoreExtended.class);
@@ -912,7 +982,7 @@ public class PulsarTestContext implements AutoCloseable {
         }
     }
 
-    @NotNull
+    @NonNull
     private static ManagedLedgerStorage createManagedLedgerClientFactory(BookKeeper bookKeeperClient,
                                                                          ManagedLedgerFactory managedLedgerFactory) {
         return new ManagedLedgerStorage() {
