@@ -19,6 +19,8 @@
 
 package org.apache.pulsar.broker.testcontext;
 
+import io.opentelemetry.sdk.autoconfigure.AutoConfiguredOpenTelemetrySdkBuilder;
+import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Supplier;
 import org.apache.pulsar.broker.BookKeeperClientFactory;
@@ -39,14 +41,15 @@ class StartableTestPulsarService extends AbstractTestPulsarService {
     private final Function<BrokerService, BrokerService> brokerServiceCustomizer;
 
     public StartableTestPulsarService(SpyConfig spyConfig, ServiceConfiguration config,
-                                      MetadataStoreExtended localMetadataStore,
-                                      MetadataStoreExtended configurationMetadataStore,
-                                      CompactionServiceFactory compactionServiceFactory,
-                                      BrokerInterceptor brokerInterceptor,
-                                      BookKeeperClientFactory bookKeeperClientFactory,
-                                      Function<BrokerService, BrokerService> brokerServiceCustomizer) {
+                                  MetadataStoreExtended localMetadataStore,
+                                  MetadataStoreExtended configurationMetadataStore,
+                                  CompactionServiceFactory compactionServiceFactory,
+                                  BrokerInterceptor brokerInterceptor,
+                                  BookKeeperClientFactory bookKeeperClientFactory,
+                                  Function<BrokerService, BrokerService> brokerServiceCustomizer,
+                                  Consumer<AutoConfiguredOpenTelemetrySdkBuilder> openTelemetrySdkBuilderCustomizer) {
         super(spyConfig, config, localMetadataStore, configurationMetadataStore, compactionServiceFactory,
-                brokerInterceptor, bookKeeperClientFactory);
+                brokerInterceptor, bookKeeperClientFactory, openTelemetrySdkBuilderCustomizer);
         this.brokerServiceCustomizer = brokerServiceCustomizer;
     }
 
@@ -59,4 +62,4 @@ class StartableTestPulsarService extends AbstractTestPulsarService {
     public Supplier<NamespaceService> getNamespaceServiceProvider() throws PulsarServerException {
         return () -> spyConfig.getNamespaceService().spy(NamespaceService.class, this);
     }
-}
+}
\ No newline at end of file
