@@ -92,6 +92,7 @@ import org.apache.pulsar.common.naming.ServiceUnitId;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
 import org.apache.pulsar.common.policies.data.ClusterData;
+import org.apache.pulsar.common.policies.data.PoliciesUtil;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import org.apache.pulsar.common.util.SecurityUtility;
@@ -112,15 +113,41 @@ import org.awaitility.Awaitility;
 import org.awaitility.reflect.WhiteboxImpl;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import org.testng.ITest;
+import org.testng.SkipException;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
+import org.testng.annotations.DataProvider;
+import org.testng.annotations.Factory;
 import org.testng.annotations.Test;
 
 @Test(groups = "broker-api")
-public class BrokerServiceLookupTest extends ProducerConsumerBase {
+public class BrokerServiceLookupTest extends ProducerConsumerBase implements ITest {
     private static final Logger log = LoggerFactory.getLogger(BrokerServiceLookupTest.class);
+    private String testName;
+
+    @DataProvider
+    private static Object[] booleanValues() {
+        return new Object[]{ true, false };
+    }
+
+    @Factory(dataProvider = "booleanValues")
+    public BrokerServiceLookupTest(boolean useTestZookeeper) {
+        // when set to true, TestZKServer is used which is a real ZooKeeper implementation
+        this.useTestZookeeper = useTestZookeeper;
+    }
+
+    @Override
+    public String getTestName() {
+        return testName;
+    }
 
     @BeforeMethod
+    public void applyTestName(Method method) {
+        testName = method.getName() + " with " + (useTestZookeeper ? "TestZKServer" : "MockZooKeeper");
+    }
+
+    @BeforeMethod(dependsOnMethods = "setTestMethodName")
     @Override
     protected void setup() throws Exception {
         conf.setDefaultNumberOfNamespaceBundles(1);
@@ -129,10 +156,43 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         producerBaseSetup();
     }
 
+    @Override
+    protected void doInitConf() throws Exception {
+        super.doInitConf();
+        switch (methodName) {
+            case "testMultipleBrokerDifferentClusterLookup" -> {
+                conf.setAuthenticationEnabled(true);
+            }
+            case "testWebserviceServiceTls" -> {
+                // broker1 with tls enabled
+                conf.setBrokerServicePortTls(Optional.of(0));
+                conf.setWebServicePortTls(Optional.of(0));
+                conf.setTlsTrustCertsFilePath(CA_CERT_FILE_PATH);
+                conf.setTlsRequireTrustedClientCertOnConnect(true);
+                conf.setTlsCertificateFilePath(BROKER_CERT_FILE_PATH);
+                conf.setTlsKeyFilePath(BROKER_KEY_FILE_PATH);
+                conf.setNumExecutorThreadPoolSize(5);
+                // Not in use, and because TLS is not configured, it will fail to start
+                conf.setSystemTopicEnabled(false);
+            }
+            case "testSkipSplitBundleIfOnlyOneBroker" -> {
+                conf.setDefaultNumberOfNamespaceBundles(1);
+                conf.setLoadBalancerNamespaceBundleMaxTopics(1);
+                conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
+            }
+            case "testPartitionedMetadataWithDeprecatedVersion" -> {
+                conf.setBrokerServicePortTls(Optional.empty());
+                conf.setWebServicePortTls(Optional.empty());
+                conf.setClientLibraryVersionCheckEnabled(true);
+            }
+        }
+    }
+
     @AfterMethod(alwaysRun = true)
     @Override
     protected void cleanup() throws Exception {
         internalCleanup();
+        testName = null;
     }
 
     @Override
@@ -293,9 +353,11 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
     @Test
     public void testConcurrentWriteBrokerData() throws Exception {
         Map<String, NamespaceBundleStats> map = new ConcurrentHashMap<>();
+        List<String> boundaries = PoliciesUtil.getBundles(100).getBoundaries();
         for (int i = 0; i < 100; i++) {
-            map.put("key"+ i, new NamespaceBundleStats());
+            map.put("my-property/my-ns/" + boundaries.get(i), new NamespaceBundleStats());
         }
+        BrokerService originalBrokerService = pulsar.getBrokerService();
         BrokerService brokerService = mock(BrokerService.class);
         doReturn(brokerService).when(pulsar).getBrokerService();
         doReturn(map).when(brokerService).getBundleStats();
@@ -326,6 +388,8 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         for (Future<?> future : list) {
             future.get();
         }
+        // allow proper shutdown so that resources aren't leaked
+        doReturn(originalBrokerService).when(pulsar).getBrokerService();
     }
 
     /**
@@ -373,12 +437,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         @Cleanup
         PulsarClient pulsarClient2 = PulsarClient.builder().serviceUrl(brokerServiceUrl.toString()).build();
 
-        // enable authorization: so, broker can validate cluster and redirect if finds different cluster
-        pulsar.getConfiguration().setAuthorizationEnabled(true);
-        // restart broker with authorization enabled: it initialize AuthorizationService
-        stopBroker();
-        startBroker();
-
         LoadManager loadManager2 = spy(pulsar2.getLoadManager().get());
         Field loadManagerField = NamespaceService.class.getDeclaredField("loadManager");
         loadManagerField.setAccessible(true);
@@ -415,10 +473,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         consumer.acknowledgeCumulative(msg);
         consumer.close();
         producer.close();
-
-        // disable authorization
-        pulsar.getConfiguration().setAuthorizationEnabled(false);
-        loadManager2 = null;
     }
 
     /**
@@ -536,18 +590,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         PulsarTestContext pulsarTestContext2 = createAdditionalPulsarTestContext(conf2);
         PulsarService pulsar2 = pulsarTestContext2.getPulsarService();
 
-        // restart broker1 with tls enabled
-        conf.setBrokerServicePortTls(Optional.of(0));
-        conf.setWebServicePortTls(Optional.of(0));
-        conf.setTlsTrustCertsFilePath(CA_CERT_FILE_PATH);
-        conf.setTlsRequireTrustedClientCertOnConnect(true);
-        conf.setTlsCertificateFilePath(BROKER_CERT_FILE_PATH);
-        conf.setTlsKeyFilePath(BROKER_KEY_FILE_PATH);
-        conf.setNumExecutorThreadPoolSize(5);
-        // Not in use, and because TLS is not configured, it will fail to start
-        conf.setSystemTopicEnabled(false);
-        stopBroker();
-        startBroker();
         pulsar.getLoadManager().get().writeLoadReportOnZookeeper();
         pulsar2.getLoadManager().get().writeLoadReportOnZookeeper();
 
@@ -751,11 +793,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
             conf2.setLoadBalancerAutoUnloadSplitBundlesEnabled(true);
             conf2.setLoadBalancerNamespaceBundleMaxTopics(1);
 
-            // configure broker-1 with ModularLoadManager
-            stopBroker();
-            conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
-            startBroker();
-
             @Cleanup
             PulsarTestContext pulsarTestContext2 = createAdditionalPulsarTestContext(conf2);
             PulsarService pulsar2 = pulsarTestContext2.getPulsarService();
@@ -873,12 +910,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         final String topicName1 = BrokerTestUtil.newUniqueName("persistent://" + namespace + "/tp_");
         final String topicName2 = BrokerTestUtil.newUniqueName("persistent://" + namespace + "/tp_");
         try {
-            // configure broker with ModularLoadManager.
-            stopBroker();
-            conf.setDefaultNumberOfNamespaceBundles(1);
-            conf.setLoadBalancerNamespaceBundleMaxTopics(1);
-            conf.setLoadManagerClassName(ModularLoadManagerImpl.class.getName());
-            startBroker();
             final ModularLoadManagerWrapper modularLoadManagerWrapper =
                     (ModularLoadManagerWrapper) pulsar.getLoadManager().get();
             final ModularLoadManagerImpl modularLoadManager =
@@ -1031,12 +1062,6 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         admin.namespaces().createNamespace(property + "/" + cluster + "/" + namespace);
         admin.topics().createPartitionedTopic(dest.toString(), totalPartitions);
 
-        stopBroker();
-        conf.setBrokerServicePortTls(Optional.empty());
-        conf.setWebServicePortTls(Optional.empty());
-        conf.setClientLibraryVersionCheckEnabled(true);
-        startBroker();
-
         URI brokerServiceUrl = new URI(pulsar.getSafeWebServiceAddress());
 
         URL url = brokerServiceUrl.toURL();
@@ -1195,6 +1220,9 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
 
     @Test
     public void testLookupConnectionNotCloseIfGetUnloadingExOrMetadataEx() throws Exception {
+        if (useTestZookeeper) {
+            throw new SkipException("This test case depends on MockZooKeeper");
+        }
         String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
         admin.topics().createNonPartitionedTopic(tpName);
         PulsarClientImpl pulsarClientImpl = (PulsarClientImpl) pulsarClient;
@@ -1310,7 +1338,8 @@ public class BrokerServiceLookupTest extends ProducerConsumerBase {
         }
     }
 
-    @Test(timeOut = 30000)
+    // TODO: This test is disabled since it's invalid. The test fails for both TestZKServer and MockZooKeeper.
+    @Test(timeOut = 30000, enabled = false)
     public void testLookupConnectionNotCloseIfFailedToAcquireOwnershipOfBundle() throws Exception {
         String tpName = BrokerTestUtil.newUniqueName("persistent://public/default/tp");
         admin.topics().createNonPartitionedTopic(tpName);
