@@ -30,7 +30,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.regex.Pattern;
 import java.util.stream.IntStream;
@@ -63,7 +62,6 @@ import org.apache.pulsar.common.policies.data.TenantInfo;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.apache.pulsar.common.policies.data.TenantOperation;
 import org.apache.pulsar.common.policies.data.TopicOperation;
-import org.apache.pulsar.common.util.RestException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.testng.Assert;
@@ -85,6 +83,7 @@ public class PatternTopicsConsumerImplAuthTest extends ProducerConsumerBase {
         // set isTcpLookup = true, to use BinaryProtoLookupService to get topics for a pattern.
         isTcpLookup = true;
 
+        conf.setTopicLevelPoliciesEnabled(false);
         conf.setAuthenticationEnabled(true);
         conf.setAuthorizationEnabled(true);
 
@@ -331,12 +330,6 @@ public class PatternTopicsConsumerImplAuthTest extends ProducerConsumerBase {
             return CompletableFuture.completedFuture(true);
         }
 
-        @Override
-        public Boolean allowTenantOperation(
-                String tenantName, String role, TenantOperation operation, AuthenticationDataSource authData) {
-            return true;
-        }
-
         @Override
         public CompletableFuture<Boolean> allowNamespaceOperationAsync(
                 NamespaceName namespaceName, String role, NamespaceOperation operation, AuthenticationDataSource authData) {
@@ -351,16 +344,6 @@ public class PatternTopicsConsumerImplAuthTest extends ProducerConsumerBase {
             return isAuthorizedFuture;
         }
 
-        @Override
-        public Boolean allowNamespaceOperation(
-                NamespaceName namespaceName, String role, NamespaceOperation operation, AuthenticationDataSource authData) {
-            try {
-                return allowNamespaceOperationAsync(namespaceName, role, operation, authData).get();
-            } catch (InterruptedException | ExecutionException e) {
-                throw new RestException(e);
-            }
-        }
-
         @Override
         public CompletableFuture<Boolean> allowTopicOperationAsync(
                 TopicName topic, String role, TopicOperation operation, AuthenticationDataSource authData) {
@@ -375,16 +358,6 @@ public class PatternTopicsConsumerImplAuthTest extends ProducerConsumerBase {
             return isAuthorizedFuture;
         }
 
-        @Override
-        public Boolean allowTopicOperation(
-                TopicName topicName, String role, TopicOperation operation, AuthenticationDataSource authData) {
-            try {
-                return allowTopicOperationAsync(topicName, role, operation, authData).get();
-            } catch (InterruptedException | ExecutionException e) {
-                throw new RestException(e);
-            }
-        }
-
         @Override
         public CompletableFuture<Boolean> allowTopicPolicyOperationAsync(TopicName topic, String role,
                                                                          PolicyName policy, PolicyOperation operation,
@@ -399,16 +372,6 @@ public class PatternTopicsConsumerImplAuthTest extends ProducerConsumerBase {
 
             return isAuthorizedFuture;
         }
-
-        @Override
-        public Boolean allowTopicPolicyOperation(TopicName topicName, String role, PolicyName policy,
-                                                 PolicyOperation operation, AuthenticationDataSource authData) {
-            try {
-                return allowTopicPolicyOperationAsync(topicName, role, policy, operation, authData).get();
-            } catch (InterruptedException | ExecutionException e) {
-                throw new RestException(e);
-            }
-        }
     }
 
     public static class ClientAuthentication implements Authentication {
