@@ -185,7 +185,7 @@ public class UnAcknowledgedMessagesTimeoutTest extends BrokerTestBase {
         }
         long size = ((ConsumerImpl<?>) consumer).getUnAckedMessageTracker().size();
         assertEquals(size, totalMessages);
-        log.info("Comulative Ack sent for " + new String(lastMessage.getData()));
+        log.info("Cumulative Ack sent for " + new String(lastMessage.getData()));
         log.info("Message ID details " + lastMessage.getMessageId().toString());
         consumer.acknowledgeCumulative(lastMessage);
         size = ((ConsumerImpl<?>) consumer).getUnAckedMessageTracker().size();
