@@ -105,7 +105,9 @@ public abstract class MockedPulsarStandalone implements AutoCloseable {
 
     }
 
-
+    protected void enableTransaction() {
+        serviceConfiguration.setTransactionCoordinatorEnabled(true);
+    }
 
     protected void configureDefaultAuthorization() {
         serviceConfiguration.setAuthorizationEnabled(true);
@@ -191,7 +193,10 @@ public abstract class MockedPulsarStandalone implements AutoCloseable {
     public void close() throws Exception {
         if (pulsarTestContext != null) {
             pulsarTestContext.close();
+            pulsarTestContext = null;
         }
+        pulsarService = null;
+        serviceInternalAdmin = null;
     }
 
     // Utils
