@@ -19,11 +19,6 @@
 package org.apache.pulsar.admin.cli;
 
 import static org.apache.pulsar.client.admin.internal.BaseResource.getApiException;
-import com.beust.jcommander.DefaultUsageFormatter;
-import com.beust.jcommander.IUsageFormatter;
-import com.beust.jcommander.JCommander;
-import com.beust.jcommander.Parameter;
-import com.beust.jcommander.ParameterException;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -34,14 +29,12 @@ import java.util.concurrent.TimeoutException;
 import java.util.function.Supplier;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminException;
-import org.apache.pulsar.client.admin.PulsarAdminException.ConnectException;
 import org.apache.pulsar.client.admin.internal.PulsarAdminImpl;
+import picocli.CommandLine;
 
 public abstract class CmdBase {
-    protected final JCommander jcommander;
+    private final CommandLine commander;
     private final Supplier<PulsarAdmin> adminSupplier;
-    private PulsarAdmin admin;
-    private IUsageFormatter usageFormatter;
 
     /**
      * Default request timeout in milliseconds.
@@ -49,91 +42,18 @@ public abstract class CmdBase {
      */
     private static final long DEFAULT_REQUEST_TIMEOUT_MILLIS = 60000;
 
-    @Parameter(names = { "--help", "-h" }, help = true, hidden = true)
-    private boolean help = false;
-
-    public boolean isHelp() {
-        return help;
-    }
-
     public CmdBase(String cmdName, Supplier<PulsarAdmin> adminSupplier) {
         this.adminSupplier = adminSupplier;
-        jcommander = new JCommander(this);
-        usageFormatter = new CmdUsageFormatter(jcommander);
-        jcommander.setProgramName("pulsar-admin " + cmdName);
-        jcommander.setUsageFormatter(usageFormatter);
-    }
-
-    protected IUsageFormatter getUsageFormatter() {
-        if (usageFormatter == null) {
-             usageFormatter = new DefaultUsageFormatter(jcommander);
-        }
-        return usageFormatter;
-    }
-
-    private void tryShowCommandUsage() {
-        try {
-            String chosenCommand = jcommander.getParsedCommand();
-            getUsageFormatter().usage(chosenCommand);
-        } catch (Exception e) {
-            // it is caused by an invalid command, the invalid command can not be parsed
-            System.err.println("Invalid command, please use `pulsar-admin --help` to check out how to use");
-        }
+        commander = new CommandLine(this);
+        commander.setCommandName(cmdName);
     }
 
     public boolean run(String[] args) {
-        try {
-            jcommander.parse(args);
-        } catch (Exception e) {
-            System.err.println(e.getMessage());
-            System.err.println();
-            tryShowCommandUsage();
-            return false;
-        }
-
-        String cmd = jcommander.getParsedCommand();
-        if (cmd == null) {
-            jcommander.usage();
-            return help;
-        }
-
-        JCommander obj = jcommander.getCommands().get(cmd);
-        CliCommand cmdObj = (CliCommand) obj.getObjects().get(0);
-
-        if (cmdObj.isHelp()) {
-            obj.setProgramName(jcommander.getProgramName() + " " + cmd);
-            obj.usage();
-            return true;
-        }
-
-        try {
-            cmdObj.run();
-            return true;
-        } catch (ParameterException e) {
-            System.err.println(e.getMessage());
-            System.err.println();
-            return false;
-        } catch (ConnectException e) {
-            System.err.println(e.getMessage());
-            System.err.println();
-            System.err.println("Error connecting to: " + getAdmin().getServiceUrl());
-            return false;
-        } catch (PulsarAdminException e) {
-            System.err.println(e.getHttpError());
-            System.err.println();
-            System.err.println("Reason: " + e.getMessage());
-            return false;
-        } catch (Exception e) {
-            e.printStackTrace();
-            return false;
-        }
+        return commander.execute(args) == 0;
     }
 
     protected PulsarAdmin getAdmin() {
-        if (admin == null) {
-            admin = adminSupplier.get();
-        }
-        return admin;
+        return adminSupplier.get();
     }
 
     protected long getRequestTimeoutMs() {
@@ -159,7 +79,7 @@ public abstract class CmdBase {
         }
     }
 
-    static Map<String, String> parseListKeyValueMap(List<String> metadata) {
+    Map<String, String> parseListKeyValueMap(List<String> metadata) {
         Map<String, String> map = null;
         if (metadata != null && !metadata.isEmpty()) {
             map = new HashMap<>();
@@ -175,7 +95,26 @@ public abstract class CmdBase {
         return map;
     }
 
-    public JCommander getJcommander() {
-        return jcommander;
+    // Used to register the subcomand.
+    protected CommandLine getCommander() {
+        return commander;
+    }
+
+    protected void addCommand(String name, Object cmd) {
+        commander.addSubcommand(name, cmd);
+    }
+
+    protected void addCommand(String name, Object cmd, String... aliases) {
+        commander.addSubcommand(name, cmd, aliases);
+    }
+
+    protected class ParameterException extends CommandLine.ParameterException {
+        public ParameterException(String msg) {
+            super(commander, msg);
+        }
+
+        public ParameterException(String msg, Throwable e) {
+            super(commander, msg, e);
+        }
     }
 }
