@@ -19,13 +19,13 @@
 package org.apache.pulsar.client.cli;
 
 import static org.apache.commons.lang3.StringUtils.isNotBlank;
-import com.beust.jcommander.Parameter;
-import com.beust.jcommander.ParameterException;
-import com.beust.jcommander.Parameters;
 import com.fasterxml.jackson.core.JsonProcessingException;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.util.concurrent.RateLimiter;
 import com.google.gson.JsonParseException;
+import java.io.ByteArrayOutputStream;
+import java.io.EOFException;
+import java.io.IOException;
 import java.net.URI;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
@@ -35,10 +35,17 @@ import java.util.Base64;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Stream;
+import org.apache.avro.generic.GenericDatumReader;
+import org.apache.avro.generic.GenericDatumWriter;
+import org.apache.avro.io.DecoderFactory;
+import org.apache.avro.io.Encoder;
+import org.apache.avro.io.EncoderFactory;
+import org.apache.avro.io.JsonDecoder;
 import org.apache.pulsar.client.api.Authentication;
 import org.apache.pulsar.client.api.AuthenticationDataProvider;
 import org.apache.pulsar.client.api.ClientBuilder;
@@ -48,6 +55,7 @@ import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.TypedMessageBuilder;
+import org.apache.pulsar.client.api.schema.KeyValueSchema;
 import org.apache.pulsar.client.impl.schema.SchemaInfoImpl;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.schema.KeyValue;
@@ -66,13 +74,18 @@ import org.eclipse.jetty.websocket.client.ClientUpgradeRequest;
 import org.eclipse.jetty.websocket.client.WebSocketClient;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import picocli.CommandLine;
+import picocli.CommandLine.Command;
+import picocli.CommandLine.Model.CommandSpec;
+import picocli.CommandLine.Option;
+import picocli.CommandLine.Parameters;
+import picocli.CommandLine.Spec;
 
 /**
  * pulsar-client produce command implementation.
- *
  */
-@Parameters(commandDescription = "Produce messages to a specified topic")
-public class CmdProduce {
+@Command(description = "Produce messages to a specified topic")
+public class CmdProduce extends AbstractCmd {
 
     private static final Logger LOG = LoggerFactory.getLogger(PulsarClientTool.class);
     private static final int MAX_MESSAGES = 1000;
@@ -80,66 +93,71 @@ public class CmdProduce {
     private static final String KEY_VALUE_ENCODING_TYPE_SEPARATED = "separated";
     private static final String KEY_VALUE_ENCODING_TYPE_INLINE = "inline";
 
-    @Parameter(description = "TopicName", required = true)
-    private List<String> mainOptions;
+    @Parameters(description = "TopicName", arity = "1")
+    private String topic;
 
-    @Parameter(names = { "-m", "--messages" },
-               description = "Messages to send, either -m or -f must be specified. Specify -m for each message.",
-               splitter = NoSplitter.class)
+    @Option(names = { "-m", "--messages" },
+            description = "Messages to send, either -m or -f must be specified. Specify -m for each message.")
     private List<String> messages = new ArrayList<>();
 
-    @Parameter(names = { "-f", "--files" },
+    @Option(names = { "-f", "--files" },
                description = "Comma separated file paths to send, either -m or -f must be specified.")
     private List<String> messageFileNames = new ArrayList<>();
 
-    @Parameter(names = { "-n", "--num-produce" },
+    @Option(names = { "-n", "--num-produce" },
                description = "Number of times to send message(s), the count of messages/files * num-produce "
                        + "should below than " + MAX_MESSAGES + ".")
     private int numTimesProduce = 1;
 
-    @Parameter(names = { "-r", "--rate" },
+    @Option(names = { "-r", "--rate" },
                description = "Rate (in msg/sec) at which to produce,"
                        + " value 0 means to produce messages as fast as possible.")
     private double publishRate = 0;
 
-    @Parameter(names = { "-db", "--disable-batching" }, description = "Disable batch sending of messages")
+    @Option(names = { "-db", "--disable-batching" }, description = "Disable batch sending of messages")
     private boolean disableBatching = false;
 
-    @Parameter(names = { "-c",
+    @Option(names = { "-c",
             "--chunking" }, description = "Should split the message and publish in chunks if message size is "
             + "larger than allowed max size")
     private boolean chunkingAllowed = false;
 
-    @Parameter(names = { "-s", "--separator" },
+    @Option(names = { "-s", "--separator" },
                description = "Character to split messages string on default is comma")
     private String separator = ",";
 
-    @Parameter(names = { "-p", "--properties"}, description = "Properties to add, Comma separated "
+    @Option(names = { "-p", "--properties"}, description = "Properties to add, Comma separated "
             + "key=value string, like k1=v1,k2=v2.")
     private List<String> properties = new ArrayList<>();
 
-    @Parameter(names = { "-k", "--key"}, description = "message key to add ")
+    @Option(names = { "-k", "--key"}, description = "Partitioning key to add to each message")
     private String key;
+    @Option(names = { "-kvk", "--key-value-key"}, description = "Value to add as message key in KeyValue schema")
+    private String keyValueKey;
+    @Option(names = {"-kvkf", "--key-value-key-file"},
+            description = "Path to file containing the value to add as message key in KeyValue schema. "
+            + "JSON and AVRO files are supported.")
+    private String keyValueKeyFile;
 
-    @Parameter(names = { "-vs", "--value-schema"}, description = "Schema type (can be bytes,avro,json,string...)")
+    @Option(names = { "-vs", "--value-schema"}, description = "Schema type (can be bytes,avro,json,string...)")
     private String valueSchema = "bytes";
 
-    @Parameter(names = { "-ks", "--key-schema"}, description = "Schema type (can be bytes,avro,json,string...)")
+    @Option(names = { "-ks", "--key-schema"}, description = "Schema type (can be bytes,avro,json,string...)")
     private String keySchema = "string";
 
-    @Parameter(names = { "-kvet", "--key-value-encoding-type"},
+    @Option(names = { "-kvet", "--key-value-encoding-type"},
             description = "Key Value Encoding Type (it can be separated or inline)")
     private String keyValueEncodingType = null;
 
-    @Parameter(names = { "-ekn", "--encryption-key-name" }, description = "The public key name to encrypt payload")
+    @Option(names = { "-ekn", "--encryption-key-name" }, description = "The public key name to encrypt payload")
     private String encKeyName = null;
 
-    @Parameter(names = { "-ekv",
+    @Option(names = { "-ekv",
             "--encryption-key-value" }, description = "The URI of public key to encrypt payload, for example "
                     + "file:///path/to/public.key or data:application/x-pem-file;base64,*****")
     private String encKeyValue = null;
 
-    @Parameter(names = { "-dr",
+    @Option(names = { "-dr",
             "--disable-replication" }, description = "Disable geo-replication for messages.")
     private boolean disableReplication = false;
 
@@ -153,7 +171,6 @@ public class CmdProduce {
 
     /**
      * Set Pulsar client configuration.
-     *
      */
     public void updateConfig(ClientBuilder newBuilder, Authentication authentication, String serviceURL) {
         this.clientBuilder = newBuilder;
@@ -170,11 +187,19 @@ public class CmdProduce {
      *
      * @return list of message bodies
      */
-    private List<byte[]> generateMessageBodies(List<String> stringMessages, List<String> messageFileNames) {
+    static List<byte[]> generateMessageBodies(List<String> stringMessages, List<String> messageFileNames,
+                                              Schema schema) {
         List<byte[]> messageBodies = new ArrayList<>();
 
         for (String m : stringMessages) {
-            messageBodies.add(m.getBytes());
+            if (schema.getSchemaInfo().getType() == SchemaType.AVRO) {
+                // JSON TO AVRO
+                org.apache.avro.Schema avroSchema = ((Optional<org.apache.avro.Schema>) schema.getNativeSchema()).get();
+                byte[] encoded = jsonToAvro(m, avroSchema);
+                messageBodies.add(encoded);
+            } else {
+                messageBodies.add(m.getBytes());
+            }
         }
 
         try {
@@ -189,6 +214,32 @@ public class CmdProduce {
         return messageBodies;
     }
 
+    private static byte[] jsonToAvro(String m, org.apache.avro.Schema avroSchema) {
+        try {
+            GenericDatumReader<Object> reader = new GenericDatumReader<>(avroSchema);
+            JsonDecoder jsonDecoder = DecoderFactory.get().jsonDecoder(avroSchema, m);
+            GenericDatumWriter<Object> writer = new GenericDatumWriter<>(avroSchema);
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            Encoder e = EncoderFactory.get().binaryEncoder(out, null);
+            Object datum = null;
+            while (true) {
+                try {
+                    datum = reader.read(datum, jsonDecoder);
+                } catch (EOFException eofException) {
+                    break;
+                }
+                writer.write(datum, e);
+                e.flush();
+            }
+            return out.toByteArray();
+        } catch (IOException e) {
+            throw new RuntimeException("Cannot convert " + m + " to AVRO " + e.getMessage(), e);
+        }
+    }
+
+    @Spec
+    private CommandSpec commandSpec;
+
     /**
      * Run the producer.
      *
@@ -196,19 +247,18 @@ public class CmdProduce {
      * @throws Exception
      */
     public int run() throws PulsarClientException {
-        if (mainOptions.size() != 1) {
-            throw (new ParameterException("Please provide one and only one topic name."));
-        }
         if (this.numTimesProduce <= 0) {
-            throw (new ParameterException("Number of times need to be positive number."));
+            throw new CommandLine.ParameterException(commandSpec.commandLine(),
+                    "Number of times need to be positive number.");
         }
 
-        if (messages.size() > 0){
+        if (messages.size() > 0) {
             messages = messages.stream().map(str -> str.split(separator)).flatMap(Stream::of).toList();
         }
 
         if (messages.size() == 0 && messageFileNames.size() == 0) {
-            throw (new ParameterException("Please supply message content with either --messages or --files"));
+            throw new CommandLine.ParameterException(commandSpec.commandLine(),
+                    "Please supply message content with either --messages or --files");
         }
 
         if (keyValueEncodingType == null) {
@@ -219,7 +269,7 @@ public class CmdProduce {
                 case KEY_VALUE_ENCODING_TYPE_INLINE:
                     break;
                 default:
-                    throw (new ParameterException("--key-value-encoding-type "
+                    throw (new IllegalArgumentException("--key-value-encoding-type "
                             + keyValueEncodingType + " is not valid, only 'separated' or 'inline'"));
             }
         }
@@ -228,11 +278,9 @@ public class CmdProduce {
         if (totalMessages > MAX_MESSAGES) {
             String msg = "Attempting to send " + totalMessages + " messages. Please do not send more than "
                     + MAX_MESSAGES + " messages";
-            throw new ParameterException(msg);
+            throw new IllegalArgumentException(msg);
         }
 
-        String topic = this.mainOptions.get(0);
-
         if (this.serviceURL.startsWith("ws")) {
             return publishToWebSocket(topic);
         } else {
@@ -258,8 +306,10 @@ public class CmdProduce {
                 producerBuilder.defaultCryptoKeyReader(this.encKeyValue);
             }
             try (Producer<?> producer = producerBuilder.create();) {
-
-                List<byte[]> messageBodies = generateMessageBodies(this.messages, this.messageFileNames);
+                Schema<?> schemaForPayload = schema.getSchemaInfo().getType() == SchemaType.KEY_VALUE
+                        ? ((KeyValueSchema) schema).getValueSchema() : schema;
+                List<byte[]> messageBodies = generateMessageBodies(this.messages, this.messageFileNames,
+                        schemaForPayload);
                 RateLimiter limiter = (this.publishRate > 0) ? RateLimiter.create(this.publishRate) : null;
 
                 Map<String, String> kvMap = new HashMap<>();
@@ -268,6 +318,25 @@ public class CmdProduce {
                     kvMap.put(kv[0], kv[1]);
                 }
 
+                final byte[] keyValueKeyBytes;
+                if (this.keyValueKey != null) {
+                    if (keyValueEncodingType == KEY_VALUE_ENCODING_TYPE_NOT_SET) {
+                        throw new IllegalArgumentException(
+                            "Key value encoding type must be set when using --key-value-key");
+                    }
+                    keyValueKeyBytes = this.keyValueKey.getBytes(StandardCharsets.UTF_8);
+                } else if (this.keyValueKeyFile != null) {
+                    if (keyValueEncodingType == KEY_VALUE_ENCODING_TYPE_NOT_SET) {
+                        throw new IllegalArgumentException(
+                            "Key value encoding type must be set when using --key-value-key-file");
+                    }
+                    keyValueKeyBytes = Files.readAllBytes(Paths.get(this.keyValueKeyFile));
+                } else if (this.key != null) {
+                    keyValueKeyBytes = this.key.getBytes(StandardCharsets.UTF_8);
+                } else {
+                    keyValueKeyBytes = null;
+                }
+
                 for (int i = 0; i < this.numTimesProduce; i++) {
                     for (byte[] content : messageBodies) {
                         if (limiter != null) {
@@ -290,8 +359,7 @@ public class CmdProduce {
                             case KEY_VALUE_ENCODING_TYPE_SEPARATED:
                             case KEY_VALUE_ENCODING_TYPE_INLINE:
                                 KeyValue kv = new KeyValue<>(
-                                        // TODO: support AVRO encoded key
-                                        key != null ? key.getBytes(StandardCharsets.UTF_8) : null,
+                                        keyValueKeyBytes,
                                         content);
                                 message.value(kv);
                                 break;
@@ -341,7 +409,7 @@ public class CmdProduce {
         Schema<?> base;
         switch (schema) {
             case "string":
-                base =  Schema.STRING;
+                base = Schema.STRING;
                 break;
             case "bytes":
                 // no need for wrappers
@@ -401,7 +469,7 @@ public class CmdProduce {
         try {
             if (authentication != null) {
                 authentication.start();
-                AuthenticationDataProvider authData = authentication.getAuthData(produceUri.getHost());
+                AuthenticationDataProvider authData = authentication.getAuthData();
                 if (authData.hasDataForHttp()) {
                     for (Map.Entry<String, String> kv : authData.getHttpHeaders()) {
                         produceRequest.setHeader(kv.getKey(), kv.getValue());
@@ -432,7 +500,7 @@ public class CmdProduce {
         }
 
         try {
-            List<byte[]> messageBodies = generateMessageBodies(this.messages, this.messageFileNames);
+            List<byte[]> messageBodies = generateMessageBodies(this.messages, this.messageFileNames, Schema.BYTES);
             RateLimiter limiter = (this.publishRate > 0) ? RateLimiter.create(this.publishRate) : null;
             for (int i = 0; i < this.numTimesProduce; i++) {
                 int index = i * 10;
