@@ -20,13 +20,19 @@ package org.apache.pulsar.client.cli;
 
 
 import static org.testng.Assert.assertEquals;
-
+import org.apache.avro.generic.GenericDatumReader;
+import org.apache.avro.generic.GenericRecord;
+import org.apache.avro.io.DecoderFactory;
+import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.schema.KeyValueSchema;
 import org.apache.pulsar.common.schema.KeyValueEncodingType;
 import org.apache.pulsar.common.schema.SchemaType;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
+import java.util.Collections;
+import java.util.List;
+
 
 public class TestCmdProduce {
 
@@ -72,4 +78,26 @@ public class TestCmdProduce {
         assertEquals(SchemaType.JSON, composite2.getKeySchema().getSchemaInfo().getType());
         assertEquals(SchemaType.AVRO, composite2.getValueSchema().getSchemaInfo().getType());
     }
+
+    @Test
+    public void generateAvroMessageBodies() throws Exception {
+
+        Schema<?> schema = CmdProduce.buildSchema(
+                null,
+                "avro:{\"type\": \"record\",\"namespace\": \"com.example\",\"name\": \"FullName\", \"fields\": [" +
+                        "{ \"name\": \"a\", \"type\": \"string\" }," +
+                        "{ \"name\": \"b\", \"type\": \"int\" }" +
+                        "]}",
+                "");
+
+        List<byte[]> bytes = CmdProduce.generateMessageBodies(List.of("{\"a\":\"stringValue\",\"b\":123}"), Collections.emptyList(), schema);
+        assertEquals(bytes.size(), 1);
+
+        org.apache.avro.Schema avro = (org.apache.avro.Schema) schema.getNativeSchema().get();
+        GenericDatumReader<GenericRecord> reader = new GenericDatumReader<>(avro);
+        GenericRecord record = reader.read(null, DecoderFactory.get().binaryDecoder(bytes.get(0), null));
+        assertEquals("stringValue", record.get("a").toString());
+        assertEquals(123, record.get("b"));
+
+    }
 }
