@@ -93,8 +93,7 @@ public class HttpClient implements Closeable {
         if ("https".equals(serviceNameResolver.getServiceUri().getServiceName())) {
             try {
                 // Set client key and certificate if available
-                AuthenticationDataProvider authData =
-                        authentication.getAuthData(serviceNameResolver.resolveHostUri().getHost());
+                AuthenticationDataProvider authData = authentication.getAuthData();
 
                 if (conf.isUseKeyStoreTls()) {
                     SSLContext sslCtx = null;
