@@ -20,11 +20,15 @@ package org.apache.pulsar.client.impl.conf;
 
 import com.fasterxml.jackson.annotation.JsonIgnore;
 import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
+import io.opentelemetry.api.OpenTelemetry;
 import io.swagger.annotations.ApiModelProperty;
 import java.io.Serializable;
 import java.net.InetSocketAddress;
 import java.net.URI;
 import java.time.Clock;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
@@ -361,6 +365,13 @@ public class ClientConfigurationData implements Serializable, Cloneable {
     )
     private int dnsLookupBindPort = 0;
 
+    @ApiModelProperty(
+            name = "dnsServerAddresses",
+            value = "The Pulsar client dns lookup server address"
+    )
+    @SuppressFBWarnings({"EI_EXPOSE_REP2", "EI_EXPOSE_REP"})
+    private List<InetSocketAddress> dnsServerAddresses = new ArrayList<>();
+
     // socks5
     @ApiModelProperty(
             name = "socks5ProxyAddress",
@@ -387,6 +398,8 @@ public class ClientConfigurationData implements Serializable, Cloneable {
     )
     private String description;
 
+    private transient OpenTelemetry openTelemetry;
+
     /**
      * Gets the authentication settings for the client.
      *
