@@ -22,12 +22,22 @@ import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyLong;
 import static org.mockito.ArgumentMatchers.anyString;
+import static org.mockito.Mockito.doAnswer;
+import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
+import io.netty.buffer.ByteBuf;
+import io.netty.channel.Channel;
+import io.netty.channel.ChannelFuture;
 import io.netty.channel.ChannelHandlerContext;
 import io.netty.channel.EventLoop;
+import io.netty.util.ReferenceCountUtil;
 import io.netty.util.Timer;
+import io.netty.util.concurrent.EventExecutor;
+import io.netty.util.concurrent.Future;
+import io.netty.util.concurrent.GenericFutureListener;
+import io.netty.util.concurrent.SucceededFuture;
 import java.net.SocketAddress;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutorService;
@@ -75,11 +85,7 @@ class ClientTestFixtures {
     }
 
     static PulsarClientImpl mockClientCnx(PulsarClientImpl clientMock) {
-        ClientCnx clientCnxMock = mock(ClientCnx.class, Mockito.RETURNS_DEEP_STUBS);
-        when(clientCnxMock.ctx()).thenReturn(mock(ChannelHandlerContext.class));
-        when(clientCnxMock.sendRequestWithId(any(), anyLong()))
-                .thenReturn(CompletableFuture.completedFuture(mock(ProducerResponse.class)));
-        when(clientCnxMock.channel().remoteAddress()).thenReturn(mock(SocketAddress.class));
+        ClientCnx clientCnxMock = mockClientCnx();
         when(clientMock.getConnection(any())).thenReturn(CompletableFuture.completedFuture(clientCnxMock));
         when(clientMock.getConnection(anyString())).thenReturn(CompletableFuture.completedFuture(clientCnxMock));
         when(clientMock.getConnection(anyString(), anyInt()))
@@ -94,6 +100,82 @@ class ClientTestFixtures {
         return clientMock;
     }
 
+    public static ClientCnx mockClientCnx() {
+        ClientCnx clientCnxMock = mock(ClientCnx.class, Mockito.RETURNS_DEEP_STUBS);
+        ChannelHandlerContext ctx = mockChannelHandlerContext();
+        doReturn(ctx).when(clientCnxMock).ctx();
+        doAnswer(invocation -> {
+            ByteBuf buf = invocation.getArgument(0);
+            buf.release();
+            return CompletableFuture.completedFuture(mock(ProducerResponse.class));
+        }).when(clientCnxMock).sendRequestWithId(any(), anyLong());
+        when(clientCnxMock.channel().remoteAddress()).thenReturn(mock(SocketAddress.class));
+        return clientCnxMock;
+    }
+
+    /**
+     * Mock a ChannelHandlerContext where write and writeAndFlush are always successful.
+     * This might not be suitable for all tests.
+     *
+     * @return a mocked ChannelHandlerContext
+     */
+    public static ChannelHandlerContext mockChannelHandlerContext() {
+        ChannelHandlerContext ctx = mock(ChannelHandlerContext.class);
+
+        // return an empty channel mock from ctx.channel()
+        Channel channel = mock(Channel.class);
+        when(ctx.channel()).thenReturn(channel);
+
+        // handle write and writeAndFlush methods so that the input message is released
+
+        // create a listener future that is returned from write and writeAndFlush
+        // that immediately completes the listener in the calling thread as if it was successful
+        ChannelFuture listenerFuture = mock(ChannelFuture.class);
+        Future<Void> succeededFuture = createSucceededFuture();
+        doAnswer(invocation -> {
+            GenericFutureListener<Future<Void>> listener = invocation.getArgument(0);
+            listener.operationComplete(succeededFuture);
+            return listenerFuture;
+        }).when(listenerFuture).addListener(any());
+
+        // handle write and writeAndFlush methods so that the input message is released
+        doAnswer(invocation -> {
+            Object msg = invocation.getArgument(0);
+            ReferenceCountUtil.release(msg);
+            return listenerFuture;
+        }).when(ctx).write(any(), any());
+        doAnswer(invocation -> {
+            Object msg = invocation.getArgument(0);
+            ReferenceCountUtil.release(msg);
+            return listenerFuture;
+        }).when(ctx).writeAndFlush(any(), any());
+        doAnswer(invocation -> {
+            Object msg = invocation.getArgument(0);
+            ReferenceCountUtil.release(msg);
+            return listenerFuture;
+        }).when(ctx).writeAndFlush(any());
+
+        return ctx;
+    }
+
+    public static Future<Void> createSucceededFuture() {
+        EventExecutor eventExecutor = mockEventExecutor();
+        // create a succeeded future that is returned from the listener, listeners will run in the calling thread
+        // using the mocked EventExecutor
+        SucceededFuture<Void> succeededFuture = new SucceededFuture<>(eventExecutor, null);
+        return succeededFuture;
+    }
+
+    public static EventExecutor mockEventExecutor() {
+        // mock an EventExecutor that runs the listener in the calling thread
+        EventExecutor eventExecutor = mock(EventExecutor.class);
+        doAnswer(invocation -> {
+            invocation.getArgument(0, Runnable.class).run();
+            return null;
+        }).when(eventExecutor).execute(any(Runnable.class));
+        return eventExecutor;
+    }
+
     static <T> CompletableFuture<T> createDelayedCompletedFuture(T result, int delayMillis) {
         CompletableFuture<T> future = new CompletableFuture<>();
         SCHEDULER.schedule(() -> future.complete(result), delayMillis, TimeUnit.MILLISECONDS);
