@@ -32,7 +32,7 @@ import io.netty.util.concurrent.DefaultThreadFactory;
 import java.time.Duration;
 import java.util.HashSet;
 import java.util.concurrent.TimeUnit;
-
+import lombok.Cleanup;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.MessageIdAdv;
 import org.apache.pulsar.client.impl.conf.ConsumerConfigurationData;
@@ -49,6 +49,7 @@ public class UnAckedMessageTrackerTest  {
         ConnectionPool connectionPool = mock(ConnectionPool.class);
         when(client.instrumentProvider()).thenReturn(InstrumentProvider.NOOP);
         when(client.getCnxPool()).thenReturn(connectionPool);
+        @Cleanup("stop")
         Timer timer = new HashedWheelTimer(new DefaultThreadFactory("pulsar-timer", Thread.currentThread().isDaemon()),
                 1, TimeUnit.MILLISECONDS);
         when(client.timer()).thenReturn(timer);
@@ -80,8 +81,6 @@ public class UnAckedMessageTrackerTest  {
         assertTrue(tracker.remove(mid));
         assertTrue(tracker.isEmpty());
         assertEquals(tracker.size(), 0);
-
-        timer.stop();
     }
 
     @Test
@@ -90,6 +89,7 @@ public class UnAckedMessageTrackerTest  {
         ConnectionPool connectionPool = mock(ConnectionPool.class);
         when(client.instrumentProvider()).thenReturn(InstrumentProvider.NOOP);
         when(client.getCnxPool()).thenReturn(connectionPool);
+        @Cleanup("stop")
         Timer timer = new HashedWheelTimer(new DefaultThreadFactory("pulsar-timer", Thread.currentThread().isDaemon()),
                 1, TimeUnit.MILLISECONDS);
         when(client.timer()).thenReturn(timer);
@@ -127,8 +127,6 @@ public class UnAckedMessageTrackerTest  {
 
         // Assert that all chunk message ID are removed from unAckedChunkedMessageIdSequenceMap
         assertEquals(consumer.unAckedChunkedMessageIdSequenceMap.size(), 0);
-
-        timer.stop();
     }
 
 }
