@@ -135,6 +135,8 @@ class ContextImpl implements Context, SinkContext, SourceContext, AutoCloseable
 
     private final Function.FunctionDetails.ComponentType componentType;
 
+    private final java.util.function.Consumer<Throwable> fatalHandler;
+
     private final ProducerCache producerCache;
     private final boolean useThreadLocalProducers;
 
@@ -143,7 +145,7 @@ class ContextImpl implements Context, SinkContext, SourceContext, AutoCloseable
                        String[] metricsLabels,
                        Function.FunctionDetails.ComponentType componentType, ComponentStatsManager statsManager,
                        StateManager stateManager, PulsarAdmin pulsarAdmin, ClientBuilder clientBuilder,
-                       ProducerCache producerCache) throws PulsarClientException {
+                       java.util.function.Consumer<Throwable> fatalHandler, ProducerCache producerCache) {
         this.config = config;
         this.logger = logger;
         this.clientBuilder = clientBuilder;
@@ -151,6 +153,7 @@ class ContextImpl implements Context, SinkContext, SourceContext, AutoCloseable
         this.pulsarAdmin = pulsarAdmin;
         this.topicSchema = new TopicSchema(client, Thread.currentThread().getContextClassLoader());
         this.statsManager = statsManager;
+        this.fatalHandler = fatalHandler;
 
         this.producerCache = producerCache;
 
@@ -532,6 +535,11 @@ class ContextImpl implements Context, SinkContext, SourceContext, AutoCloseable
         return clientBuilder;
     }
 
+    @Override
+    public void fatal(Throwable t) {
+        fatalHandler.accept(t);
+    }
+
     private <T> Producer<T> getProducer(String topicName, Schema<T> schema) throws PulsarClientException {
         Long additionalCacheKey = useThreadLocalProducers ? Thread.currentThread().getId() : null;
         return producerCache.getOrCreateProducer(ProducerCache.CacheArea.CONTEXT_CACHE,
@@ -560,12 +568,13 @@ class ContextImpl implements Context, SinkContext, SourceContext, AutoCloseable
             String metricName = userMetricsLabelsEntry.getKey();
             String[] labels = userMetricsLabelsEntry.getValue();
             Summary.Child.Value summary = userMetricsSummary.labels(labels).get();
-            metricsMap.put(String.format("%s%s_sum", USER_METRIC_PREFIX, metricName), summary.sum);
-            metricsMap.put(String.format("%s%s_count", USER_METRIC_PREFIX, metricName), summary.count);
+            String prefix = USER_METRIC_PREFIX + metricName + "_";
+            metricsMap.put(prefix + "sum", summary.sum);
+            metricsMap.put(prefix + "count", summary.count);
             for (Map.Entry<Double, Double> entry : summary.quantiles.entrySet()) {
                 Double quantile = entry.getKey();
                 Double value = entry.getValue();
-                metricsMap.put(String.format("%s%s_%s", USER_METRIC_PREFIX, metricName, quantile), value);
+                metricsMap.put(prefix + quantile, value);
             }
         }
         return metricsMap;
