@@ -18,32 +18,17 @@
  */
 package org.apache.pulsar.functions.sink;
 
-import static org.apache.commons.lang.StringUtils.isEmpty;
 import com.google.common.annotations.VisibleForTesting;
 import java.nio.charset.StandardCharsets;
-import java.security.Security;
-import java.util.ArrayList;
 import java.util.Base64;
-import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.TimeUnit;
 import java.util.function.Function;
-import lombok.Builder;
-import lombok.Data;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.lang3.StringUtils;
-import org.apache.pulsar.client.api.BatcherBuilder;
-import org.apache.pulsar.client.api.CompressionType;
-import org.apache.pulsar.client.api.CryptoKeyReader;
-import org.apache.pulsar.client.api.HashingScheme;
 import org.apache.pulsar.client.api.MessageId;
-import org.apache.pulsar.client.api.MessageRoutingMode;
 import org.apache.pulsar.client.api.Producer;
-import org.apache.pulsar.client.api.ProducerBuilder;
-import org.apache.pulsar.client.api.ProducerCryptoFailureAction;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
@@ -52,22 +37,19 @@ import org.apache.pulsar.client.api.schema.GenericRecord;
 import org.apache.pulsar.client.api.schema.KeyValueSchema;
 import org.apache.pulsar.client.impl.schema.AutoConsumeSchema;
 import org.apache.pulsar.common.functions.ConsumerConfig;
-import org.apache.pulsar.common.functions.CryptoConfig;
 import org.apache.pulsar.common.functions.FunctionConfig;
-import org.apache.pulsar.common.functions.ProducerConfig;
 import org.apache.pulsar.common.schema.KeyValueEncodingType;
 import org.apache.pulsar.common.schema.SchemaType;
 import org.apache.pulsar.common.util.Reflections;
 import org.apache.pulsar.functions.api.Record;
 import org.apache.pulsar.functions.instance.AbstractSinkRecord;
-import org.apache.pulsar.functions.instance.FunctionResultRouter;
+import org.apache.pulsar.functions.instance.ProducerBuilderFactory;
+import org.apache.pulsar.functions.instance.ProducerCache;
 import org.apache.pulsar.functions.instance.stats.ComponentStatsManager;
 import org.apache.pulsar.functions.source.PulsarRecord;
 import org.apache.pulsar.functions.source.TopicSchema;
-import org.apache.pulsar.functions.utils.CryptoUtils;
 import org.apache.pulsar.io.core.Sink;
 import org.apache.pulsar.io.core.SinkContext;
-import org.bouncycastle.jce.provider.BouncyCastleProvider;
 
 @Slf4j
 public class PulsarSink<T> implements Sink<T> {
@@ -77,11 +59,14 @@ public class PulsarSink<T> implements Sink<T> {
     private final Map<String, String> properties;
     private final ClassLoader functionClassLoader;
     private ComponentStatsManager stats;
+    private final ProducerCache producerCache;
 
     @VisibleForTesting
     PulsarSinkProcessor<T> pulsarSinkProcessor;
 
     private final TopicSchema topicSchema;
+    private Schema<T> schema;
+    private ProducerBuilderFactory producerBuilderFactory;
 
     private interface PulsarSinkProcessor<T> {
 
@@ -93,98 +78,25 @@ public class PulsarSink<T> implements Sink<T> {
     }
 
     abstract class PulsarSinkProcessorBase implements PulsarSinkProcessor<T> {
-        protected Map<String, Producer<T>> publishProducers = new ConcurrentHashMap<>();
-        protected Schema schema;
-        protected Crypto crypto;
-
-        protected PulsarSinkProcessorBase(Schema schema, Crypto crypto) {
-            this.schema = schema;
-            this.crypto = crypto;
-        }
-
-        public Producer<T> createProducer(PulsarClient client, String topic, String producerName, Schema<T> schema)
-                throws PulsarClientException {
-            ProducerBuilder<T> builder = client.newProducer(schema)
-                    .blockIfQueueFull(true)
-                    .enableBatching(true)
-                    .batchingMaxPublishDelay(10, TimeUnit.MILLISECONDS)
-                    .hashingScheme(HashingScheme.Murmur3_32Hash) //
-                    .messageRoutingMode(MessageRoutingMode.CustomPartition)
-                    .messageRouter(FunctionResultRouter.of())
-                    // set send timeout to be infinity to prevent potential deadlock with consumer
-                    // that might happen when consumer is blocked due to unacked messages
-                    .sendTimeout(0, TimeUnit.SECONDS)
-                    .topic(topic);
-            if (producerName != null) {
-                builder.producerName(producerName);
-            }
-            if (pulsarSinkConfig.getProducerConfig() != null) {
-                ProducerConfig producerConfig = pulsarSinkConfig.getProducerConfig();
-                if (producerConfig.getCompressionType() != null) {
-                    builder.compressionType(producerConfig.getCompressionType());
-                } else {
-                    builder.compressionType(CompressionType.LZ4);
-                }
-                if (producerConfig.getMaxPendingMessages() != 0) {
-                    builder.maxPendingMessages(producerConfig.getMaxPendingMessages());
-                }
-                if (producerConfig.getMaxPendingMessagesAcrossPartitions() != 0) {
-                    builder.maxPendingMessagesAcrossPartitions(producerConfig.getMaxPendingMessagesAcrossPartitions());
-                }
-                if (producerConfig.getCryptoConfig() != null) {
-                    builder.cryptoKeyReader(crypto.keyReader);
-                    builder.cryptoFailureAction(crypto.failureAction);
-                    for (String encryptionKeyName : crypto.getEncryptionKeys()) {
-                        builder.addEncryptionKey(encryptionKeyName);
-                    }
-                }
-                if (producerConfig.getBatchBuilder() != null) {
-                    if (producerConfig.getBatchBuilder().equals("KEY_BASED")) {
-                        builder.batcherBuilder(BatcherBuilder.KEY_BASED);
-                    } else {
-                        builder.batcherBuilder(BatcherBuilder.DEFAULT);
-                    }
-                }
-            }
-            return builder.properties(properties).create();
-        }
-
         protected Producer<T> getProducer(String destinationTopic, Schema schema) {
-            return getProducer(destinationTopic, null, destinationTopic, schema);
+            return getProducer(destinationTopic, schema, null, null);
         }
 
-        protected Producer<T> getProducer(String producerId, String producerName, String topicName, Schema schema) {
-            return publishProducers.computeIfAbsent(producerId, s -> {
-                try {
-                    log.info("Initializing producer {} on topic {} with schema {}",
-                        producerName, topicName, schema);
-                    Producer<T> producer = createProducer(
-                            client,
-                            topicName,
-                            producerName,
-                            schema != null ? schema : this.schema);
-                    log.info("Initialized producer {} on topic {} with schema {}: {} -> {}",
-                        producerName, topicName, schema, producerId, producer);
-                    return producer;
-                } catch (PulsarClientException e) {
-                    log.error("Failed to create Producer while doing user publish", e);
-                    throw new RuntimeException(e);
-                }
-            });
+        protected Producer<T> getProducer(String topicName, Schema schema, String producerName, String partitionId) {
+            return producerCache.getOrCreateProducer(ProducerCache.CacheArea.SINK_RECORD_CACHE, topicName, partitionId,
+                    () -> {
+                        Producer<T> producer = createProducer(topicName, schema, producerName);
+                        log.info(
+                                "Initialized producer with name '{}' on topic '{}' with schema {} partitionId {} "
+                                        + "-> {}",
+                                producerName, topicName, schema, partitionId, producer);
+                        return producer;
+                    });
         }
 
         @Override
         public void close() throws Exception {
-            List<CompletableFuture<Void>> closeFutures = new ArrayList<>(publishProducers.size());
-            for (Map.Entry<String, Producer<T>> entry : publishProducers.entrySet()) {
-                Producer<T> producer = entry.getValue();
-                closeFutures.add(producer.closeAsync());
-            }
-            try {
-                org.apache.pulsar.common.util.FutureUtil.waitForAll(closeFutures);
-            } catch (Exception e) {
-                log.warn("Failed to close all the producers", e);
-            }
+            // no op
         }
 
         public Function<Throwable, Void> getPublishErrorHandler(AbstractSinkRecord<T> record, boolean failSource) {
@@ -218,17 +130,10 @@ public class PulsarSink<T> implements Sink<T> {
 
     @VisibleForTesting
     class PulsarSinkAtMostOnceProcessor extends PulsarSinkProcessorBase {
-        public PulsarSinkAtMostOnceProcessor(Schema schema, Crypto crypto) {
-            super(schema, crypto);
+        public PulsarSinkAtMostOnceProcessor() {
             if (!(schema instanceof AutoConsumeSchema)) {
                 // initialize default topic
-                try {
-                    publishProducers.put(pulsarSinkConfig.getTopic(),
-                        createProducer(client, pulsarSinkConfig.getTopic(), null, schema));
-                } catch (PulsarClientException e) {
-                    log.error("Failed to create Producer while doing user publish", e);
-                    throw new RuntimeException(e);
-                }
+                getProducer(pulsarSinkConfig.getTopic(), schema);
             } else {
                 if (log.isDebugEnabled()) {
                     log.debug("The Pulsar producer is not initialized until the first record is"
@@ -270,10 +175,6 @@ public class PulsarSink<T> implements Sink<T> {
 
     @VisibleForTesting
     class PulsarSinkAtLeastOnceProcessor extends PulsarSinkAtMostOnceProcessor {
-        public PulsarSinkAtLeastOnceProcessor(Schema schema, Crypto crypto) {
-            super(schema, crypto);
-        }
-
         @Override
         public void sendOutputMessage(TypedMessageBuilder<T> msg, AbstractSinkRecord<T> record) {
             msg.sendAsync()
@@ -284,11 +185,6 @@ public class PulsarSink<T> implements Sink<T> {
 
     @VisibleForTesting
     class PulsarSinkManualProcessor extends PulsarSinkAtMostOnceProcessor {
-
-        public PulsarSinkManualProcessor(Schema schema, Crypto crypto) {
-            super(schema, crypto);
-        }
-
         @Override
         public void sendOutputMessage(TypedMessageBuilder<T> msg, AbstractSinkRecord<T> record) {
             super.sendOutputMessage(msg, record);
@@ -297,11 +193,6 @@ public class PulsarSink<T> implements Sink<T> {
 
     @VisibleForTesting
     class PulsarSinkEffectivelyOnceProcessor extends PulsarSinkProcessorBase {
-
-        public PulsarSinkEffectivelyOnceProcessor(Schema schema, Crypto crypto) {
-            super(schema, crypto);
-        }
-
         @Override
         public TypedMessageBuilder<T> newMessage(AbstractSinkRecord<T> record) {
             if (!record.getPartitionId().isPresent()) {
@@ -315,13 +206,10 @@ public class PulsarSink<T> implements Sink<T> {
                 // we must use the destination topic schema
                 schemaToWrite = schema;
             }
-            Producer<T> producer = getProducer(
-                    String.format("%s-%s", record.getDestinationTopic().orElse(pulsarSinkConfig.getTopic()),
-                            record.getPartitionId().get()),
-                    record.getPartitionId().get(),
-                    record.getDestinationTopic().orElse(pulsarSinkConfig.getTopic()),
-                    schemaToWrite
-            );
+            String topicName = record.getDestinationTopic().orElse(pulsarSinkConfig.getTopic());
+            String partitionId = record.getPartitionId().get();
+            String producerName = partitionId;
+            Producer<T> producer = getProducer(topicName, schemaToWrite, producerName, partitionId);
             if (schemaToWrite != null) {
                 return producer.newMessage(schemaToWrite);
             } else {
@@ -346,43 +234,41 @@ public class PulsarSink<T> implements Sink<T> {
     }
 
     public PulsarSink(PulsarClient client, PulsarSinkConfig pulsarSinkConfig, Map<String, String> properties,
-                      ComponentStatsManager stats, ClassLoader functionClassLoader) {
+                      ComponentStatsManager stats, ClassLoader functionClassLoader, ProducerCache producerCache) {
         this.client = client;
         this.pulsarSinkConfig = pulsarSinkConfig;
         this.topicSchema = new TopicSchema(client, functionClassLoader);
         this.properties = properties;
         this.stats = stats;
         this.functionClassLoader = functionClassLoader;
+        this.producerCache = producerCache;
     }
 
     @Override
     public void open(Map<String, Object> config, SinkContext sinkContext) throws Exception {
         log.info("Opening pulsar sink with config: {}", pulsarSinkConfig);
 
-        Schema<T> schema = initializeSchema();
+        schema = initializeSchema();
         if (schema == null) {
             log.info("Since output type is null, not creating any real sink");
             return;
         }
-
-        Crypto crypto = initializeCrypto();
-        if (crypto == null) {
-            log.info("crypto key reader is not provided, not enabling end to end encryption");
-        }
+        producerBuilderFactory =
+                new ProducerBuilderFactory(client, pulsarSinkConfig.getProducerConfig(), functionClassLoader, null);
 
         FunctionConfig.ProcessingGuarantees processingGuarantees = this.pulsarSinkConfig.getProcessingGuarantees();
         switch (processingGuarantees) {
             case ATMOST_ONCE:
-                this.pulsarSinkProcessor = new PulsarSinkAtMostOnceProcessor(schema, crypto);
+                this.pulsarSinkProcessor = new PulsarSinkAtMostOnceProcessor();
                 break;
             case ATLEAST_ONCE:
-                this.pulsarSinkProcessor = new PulsarSinkAtLeastOnceProcessor(schema, crypto);
+                this.pulsarSinkProcessor = new PulsarSinkAtLeastOnceProcessor();
                 break;
             case EFFECTIVELY_ONCE:
-                this.pulsarSinkProcessor = new PulsarSinkEffectivelyOnceProcessor(schema, crypto);
+                this.pulsarSinkProcessor = new PulsarSinkEffectivelyOnceProcessor();
                 break;
             case MANUAL:
-                this.pulsarSinkProcessor = new PulsarSinkManualProcessor(schema, crypto);
+                this.pulsarSinkProcessor = new PulsarSinkManualProcessor();
                 break;
         }
     }
@@ -427,6 +313,19 @@ public class PulsarSink<T> implements Sink<T> {
         }
     }
 
+    Producer<T> createProducer(String topicName, Schema<T> schema, String producerName) {
+        Schema<T> schemaToUse = schema != null ? schema : this.schema;
+        try {
+            log.info("Initializing producer {} on topic {} with schema {}", producerName, topicName, schemaToUse);
+            return producerBuilderFactory.createProducerBuilder(topicName, schemaToUse, producerName)
+                    .properties(properties)
+                    .create();
+        } catch (PulsarClientException e) {
+            throw new RuntimeException("Failed to create Producer for topic " + topicName
+                    + " producerName " + producerName + " schema " + schemaToUse, e);
+        }
+    }
+
     @SuppressWarnings("unchecked")
     @VisibleForTesting
     Schema<T> initializeSchema() throws ClassNotFoundException {
@@ -461,39 +360,5 @@ public class PulsarSink<T> implements Sink<T> {
         }
     }
 
-    @SuppressWarnings("unchecked")
-    @VisibleForTesting
-    Crypto initializeCrypto() throws ClassNotFoundException {
-        if (pulsarSinkConfig.getProducerConfig() == null
-                || pulsarSinkConfig.getProducerConfig().getCryptoConfig() == null
-                || isEmpty(pulsarSinkConfig.getProducerConfig().getCryptoConfig().getCryptoKeyReaderClassName())) {
-            return null;
-        }
 
-        CryptoConfig cryptoConfig = pulsarSinkConfig.getProducerConfig().getCryptoConfig();
-
-        // add provider only if it's not in the JVM
-        if (Security.getProvider(BouncyCastleProvider.PROVIDER_NAME) == null) {
-            Security.addProvider(new BouncyCastleProvider());
-        }
-
-        final String[] encryptionKeys = cryptoConfig.getEncryptionKeys();
-        Crypto.CryptoBuilder bldr = Crypto.builder()
-                .failureAction(cryptoConfig.getProducerCryptoFailureAction())
-                .encryptionKeys(encryptionKeys);
-
-        bldr.keyReader(CryptoUtils.getCryptoKeyReaderInstance(
-                cryptoConfig.getCryptoKeyReaderClassName(), cryptoConfig.getCryptoKeyReaderConfig(),
-                functionClassLoader));
-
-        return bldr.build();
-    }
-
-    @Data
-    @Builder
-    private static class Crypto {
-        private CryptoKeyReader keyReader;
-        private ProducerCryptoFailureAction failureAction;
-        private String[] encryptionKeys;
-    }
 }
