@@ -21,7 +21,6 @@ package org.apache.pulsar.io.kinesis;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
 
-import java.io.File;
 import java.io.IOException;
 import java.time.ZoneOffset;
 import java.time.ZonedDateTime;
@@ -30,7 +29,6 @@ import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
 
-import org.apache.pulsar.io.common.IOConfigUtils;
 import org.apache.pulsar.io.core.SourceContext;
 import org.mockito.Mockito;
 import org.testng.annotations.Test;
@@ -54,30 +52,6 @@ public class KinesisSourceConfigTests {
         DAY = then.getTime();
     }
 
-    @Test
-    public final void loadFromYamlFileTest() throws IOException {
-        File yamlFile = getFile("sourceConfig.yaml");
-        KinesisSourceConfig config = KinesisSourceConfig.load(yamlFile.getAbsolutePath());
-        assertNotNull(config);
-        assertEquals(config.getAwsEndpoint(), "https://some.endpoint.aws");
-        assertEquals(config.getAwsRegion(), "us-east-1");
-        assertEquals(config.getAwsKinesisStreamName(), "my-stream");
-        assertEquals(config.getAwsCredentialPluginParam(),
-                "{\"accessKey\":\"myKey\",\"secretKey\":\"my-Secret\"}");
-        assertEquals(config.getApplicationName(), "My test application");
-        assertEquals(config.getCheckpointInterval(), 30000);
-        assertEquals(config.getBackoffTime(), 4000);
-        assertEquals(config.getNumRetries(), 3);
-        assertEquals(config.getReceiveQueueSize(), 2000);
-        assertEquals(config.getInitialPositionInStream(), InitialPositionInStream.TRIM_HORIZON);
-
-        Calendar cal = Calendar.getInstance();
-        cal.setTime(config.getStartAtTime());
-        ZonedDateTime actual = ZonedDateTime.ofInstant(cal.toInstant(), ZoneOffset.UTC);
-        ZonedDateTime expected = ZonedDateTime.ofInstant(DAY.toInstant(), ZoneOffset.UTC);
-        assertEquals(actual, expected);
-    }
-
     @Test
     public final void loadFromMapTest() throws IOException {
         Map<String, Object> map = new HashMap<String, Object> ();
@@ -89,12 +63,11 @@ public class KinesisSourceConfigTests {
         map.put("backoffTime", "4000");
         map.put("numRetries", "3");
         map.put("receiveQueueSize", 2000);
-        map.put("applicationName", "My test application");
         map.put("initialPositionInStream", InitialPositionInStream.TRIM_HORIZON);
         map.put("startAtTime", DAY);
 
         SourceContext sourceContext = Mockito.mock(SourceContext.class);
-        KinesisSourceConfig config = IOConfigUtils.loadWithSecrets(map, KinesisSourceConfig.class, sourceContext);
+        KinesisSourceConfig config = KinesisSourceConfig.load(map, sourceContext);
 
         assertNotNull(config);
         assertEquals(config.getAwsEndpoint(), "https://some.endpoint.aws");
@@ -102,7 +75,7 @@ public class KinesisSourceConfigTests {
         assertEquals(config.getAwsKinesisStreamName(), "my-stream");
         assertEquals(config.getAwsCredentialPluginParam(),
                 "{\"accessKey\":\"myKey\",\"secretKey\":\"my-Secret\"}");
-        assertEquals(config.getApplicationName(), "My test application");
+        assertEquals(config.getApplicationName(), "pulsar-kinesis");
         assertEquals(config.getCheckpointInterval(), 30000);
         assertEquals(config.getBackoffTime(), 4000);
         assertEquals(config.getNumRetries(), 3);
@@ -133,7 +106,7 @@ public class KinesisSourceConfigTests {
         SourceContext sourceContext = Mockito.mock(SourceContext.class);
         Mockito.when(sourceContext.getSecret("awsCredentialPluginParam"))
                 .thenReturn("{\"accessKey\":\"myKey\",\"secretKey\":\"my-Secret\"}");
-        KinesisSourceConfig config = IOConfigUtils.loadWithSecrets(map, KinesisSourceConfig.class, sourceContext);
+        KinesisSourceConfig config = KinesisSourceConfig.load(map, sourceContext);
 
         assertNotNull(config);
         assertEquals(config.getAwsEndpoint(), "https://some.endpoint.aws");
@@ -156,19 +129,17 @@ public class KinesisSourceConfigTests {
     }
 
     @Test(expectedExceptions = IllegalArgumentException.class,
-            expectedExceptionsMessageRegExp = "empty aws-credential param")
+            expectedExceptionsMessageRegExp = "awsCredentialPluginParam cannot be null")
     public final void missingCredentialsTest() throws Exception {
         Map<String, Object> map = new HashMap<String, Object> ();
         map.put("awsEndpoint", "https://some.endpoint.aws");
         map.put("awsRegion", "us-east-1");
         map.put("awsKinesisStreamName", "my-stream");
-
-        KinesisSource source = new KinesisSource();
-        source.open(map, null);
+        KinesisSourceConfig.load(map, Mockito.mock(SourceContext.class));
     }
 
     @Test(expectedExceptions = IllegalArgumentException.class,
-            expectedExceptionsMessageRegExp = "Timestamp must be specified")
+            expectedExceptionsMessageRegExp = "When initialPositionInStream is AT_TIMESTAMP, startAtTime must be specified")
     public final void missingStartTimeTest() throws Exception {
         Map<String, Object> map = new HashMap<String, Object> ();
         map.put("awsEndpoint", "https://some.endpoint.aws");
@@ -177,13 +148,16 @@ public class KinesisSourceConfigTests {
         map.put("awsCredentialPluginParam",
                 "{\"accessKey\":\"myKey\",\"secretKey\":\"my-Secret\"}");
         map.put("initialPositionInStream", InitialPositionInStream.AT_TIMESTAMP);
-
-        KinesisSource source = new KinesisSource();
-        source.open(map, null);
+        KinesisSourceConfig.load(map, Mockito.mock(SourceContext.class));
     }
 
-    private File getFile(String name) {
-        ClassLoader classLoader = getClass().getClassLoader();
-        return new File(classLoader.getResource(name).getFile());
+    @Test(expectedExceptions = IllegalArgumentException.class)
+    public final void missCloudWatchEndpointTest() {
+        Map<String, Object> map = new HashMap<String, Object> ();
+        map.put("awsEndpoint", "https://some.endpoint.aws");
+        map.put("awsKinesisStreamName", "my-stream");
+        map.put("awsCredentialPluginParam",
+                "{\"accessKey\":\"myKey\",\"secretKey\":\"my-Secret\"}");
+        KinesisSourceConfig.load(map, Mockito.mock(SourceContext.class));
     }
 }
