@@ -18,7 +18,9 @@
  */
 package org.apache.pulsar.io.rabbitmq.sink;
 
+import org.apache.pulsar.io.core.SinkContext;
 import org.apache.pulsar.io.rabbitmq.RabbitMQSinkConfig;
+import org.mockito.Mockito;
 import org.testng.annotations.Test;
 
 import java.io.File;
@@ -71,7 +73,45 @@ public class RabbitMQSinkConfigTest {
         map.put("exchangeName", "test-exchange");
         map.put("exchangeType", "test-exchange-type");
 
-        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map);
+        SinkContext sinkContext = Mockito.mock(SinkContext.class);
+        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map, sinkContext);
+        assertNotNull(config);
+        assertEquals(config.getHost(), "localhost");
+        assertEquals(config.getPort(), Integer.parseInt("5673"));
+        assertEquals(config.getVirtualHost(), "/");
+        assertEquals(config.getUsername(), "guest");
+        assertEquals(config.getPassword(), "guest");
+        assertEquals(config.getConnectionName(), "test-connection");
+        assertEquals(config.getRequestedChannelMax(), Integer.parseInt("0"));
+        assertEquals(config.getRequestedFrameMax(), Integer.parseInt("0"));
+        assertEquals(config.getConnectionTimeout(), Integer.parseInt("60000"));
+        assertEquals(config.getHandshakeTimeout(), Integer.parseInt("10000"));
+        assertEquals(config.getRequestedHeartbeat(), Integer.parseInt("60"));
+        assertEquals(config.getExchangeName(), "test-exchange");
+        assertEquals(config.getExchangeType(), "test-exchange-type");
+    }
+
+    @Test
+    public final void loadFromMapCredentialsFromSecretTest() throws IOException {
+        Map<String, Object> map = new HashMap<>();
+        map.put("host", "localhost");
+        map.put("port", "5673");
+        map.put("virtualHost", "/");
+        map.put("connectionName", "test-connection");
+        map.put("requestedChannelMax", "0");
+        map.put("requestedFrameMax", "0");
+        map.put("connectionTimeout", "60000");
+        map.put("handshakeTimeout", "10000");
+        map.put("requestedHeartbeat", "60");
+        map.put("exchangeName", "test-exchange");
+        map.put("exchangeType", "test-exchange-type");
+
+        SinkContext sinkContext = Mockito.mock(SinkContext.class);
+        Mockito.when(sinkContext.getSecret("username"))
+                .thenReturn("guest");
+        Mockito.when(sinkContext.getSecret("password"))
+                .thenReturn("guest");
+        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map, sinkContext);
         assertNotNull(config);
         assertEquals(config.getHost(), "localhost");
         assertEquals(config.getPort(), Integer.parseInt("5673"));
@@ -105,12 +145,13 @@ public class RabbitMQSinkConfigTest {
         map.put("exchangeName", "test-exchange");
         map.put("exchangeType", "test-exchange-type");
 
-        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map);
+        SinkContext sinkContext = Mockito.mock(SinkContext.class);
+        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map, sinkContext);
         config.validate();
     }
 
-    @Test(expectedExceptions = NullPointerException.class,
-        expectedExceptionsMessageRegExp = "exchangeName property not set.")
+    @Test(expectedExceptions = IllegalArgumentException.class,
+        expectedExceptionsMessageRegExp = "exchangeName cannot be null")
     public final void missingExchangeValidateTest() throws IOException {
         Map<String, Object> map = new HashMap<>();
         map.put("host", "localhost");
@@ -126,7 +167,8 @@ public class RabbitMQSinkConfigTest {
         map.put("requestedHeartbeat", "60");
         map.put("exchangeType", "test-exchange-type");
 
-        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map);
+        SinkContext sinkContext = Mockito.mock(SinkContext.class);
+        RabbitMQSinkConfig config = RabbitMQSinkConfig.load(map, sinkContext);
         config.validate();
     }
 
