@@ -25,6 +25,7 @@ import com.github.benmanes.caffeine.cache.AsyncLoadingCache;
 import com.github.benmanes.caffeine.cache.Caffeine;
 import com.google.common.annotations.VisibleForTesting;
 import java.io.IOException;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
@@ -50,12 +51,15 @@ import org.apache.pulsar.metadata.api.MetadataStoreException.BadVersionException
 import org.apache.pulsar.metadata.api.MetadataStoreException.ContentDeserializationException;
 import org.apache.pulsar.metadata.api.MetadataStoreException.NotFoundException;
 import org.apache.pulsar.metadata.api.Notification;
+import org.apache.pulsar.metadata.api.extended.CreateOption;
+import org.apache.pulsar.metadata.api.extended.MetadataStoreExtended;
 import org.apache.pulsar.metadata.impl.AbstractMetadataStore;
 
 @Slf4j
 public class MetadataCacheImpl<T> implements MetadataCache<T>, Consumer<Notification> {
     @Getter
     private final MetadataStore store;
+    private final MetadataStoreExtended storeExtended;
     private final MetadataSerde<T> serde;
     private final ScheduledExecutorService executor;
     private final MetadataCacheConfig cacheConfig;
@@ -75,6 +79,11 @@ public class MetadataCacheImpl<T> implements MetadataCache<T>, Consumer<Notifica
     public MetadataCacheImpl(MetadataStore store, MetadataSerde<T> serde, MetadataCacheConfig cacheConfig,
                              ScheduledExecutorService executor) {
         this.store = store;
+        if (store instanceof MetadataStoreExtended) {
+            this.storeExtended = (MetadataStoreExtended) store;
+        } else {
+            this.storeExtended = null;
+        }
         this.serde = serde;
         this.cacheConfig = cacheConfig;
         this.executor = executor;
@@ -253,6 +262,21 @@ public class MetadataCacheImpl<T> implements MetadataCache<T>, Consumer<Notifica
         return future;
     }
 
+    @Override
+    public CompletableFuture<Void> put(String path, T value, EnumSet<CreateOption> options) {
+        final byte[] bytes;
+        try {
+            bytes = serde.serialize(path, value);
+        } catch (IOException e) {
+            return CompletableFuture.failedFuture(e);
+        }
+        if (storeExtended != null) {
+            return storeExtended.put(path, bytes, Optional.empty(), options).thenAccept(__ -> refresh(path));
+        } else {
+            return store.put(path, bytes, Optional.empty()).thenAccept(__ -> refresh(path));
+        }
+    }
+
     @Override
     public CompletableFuture<Void> delete(String path) {
         return store.delete(path, Optional.empty());
