@@ -52,7 +52,7 @@ public abstract class AbstractBatchedMetadataStore extends AbstractMetadataStore
     private final int maxDelayMillis;
     private final int maxOperations;
     private final int maxSize;
-    private final MetadataEventSynchronizer synchronizer;
+    private MetadataEventSynchronizer synchronizer;
     private final BatchMetadataStoreStats batchMetadataStoreStats;
 
     protected AbstractBatchedMetadataStore(MetadataStoreConfig conf) {
@@ -75,8 +75,7 @@ public abstract class AbstractBatchedMetadataStore extends AbstractMetadataStore
         }
 
         // update synchronizer and register sync listener
-        synchronizer = conf.getSynchronizer();
-        registerSyncListener(Optional.ofNullable(synchronizer));
+        updateMetadataEventSynchronizer(conf.getSynchronizer());
         this.batchMetadataStoreStats =
                 new BatchMetadataStoreStats(metadataStoreName, executor);
     }
@@ -135,7 +134,7 @@ public abstract class AbstractBatchedMetadataStore extends AbstractMetadataStore
     }
 
     @Override
-    protected final CompletableFuture<List<String>> getChildrenFromStore(String path) {
+    public final CompletableFuture<List<String>> getChildrenFromStore(String path) {
         OpGetChildren op = new OpGetChildren(path);
         enqueue(readOps, op);
         return op.getFuture();
@@ -161,6 +160,12 @@ public abstract class AbstractBatchedMetadataStore extends AbstractMetadataStore
         return Optional.ofNullable(synchronizer);
     }
 
+    @Override
+    public void updateMetadataEventSynchronizer(MetadataEventSynchronizer synchronizer) {
+        this.synchronizer = synchronizer;
+        registerSyncListener(Optional.ofNullable(synchronizer));
+    }
+
     private void enqueue(MessagePassingQueue<MetadataOp> queue, MetadataOp op) {
         if (enabled) {
             if (!queue.offer(op)) {
