@@ -21,16 +21,23 @@ package org.apache.pulsar.proxy.server;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.net.URI;
 import java.nio.ByteBuffer;
 import java.util.Base64;
 import java.util.Map;
 import java.util.Optional;
+import java.util.Properties;
 import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.Future;
 import java.util.function.Consumer;
 import lombok.Cleanup;
+import lombok.SneakyThrows;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
 import org.apache.pulsar.client.api.Authentication;
 import org.apache.pulsar.client.api.Producer;
@@ -50,17 +57,38 @@ import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
 public class ProxyServiceStarterTest extends MockedPulsarServiceBaseTest {
-
-    public static final String[] ARGS = new String[]{"-c", "./src/test/resources/proxy.conf"};
-
     protected ProxyServiceStarter serviceStarter;
     protected String serviceUrl;
+    private static File proxyConfFileForTests;
+
+    @SneakyThrows
+    public static String[] getArgs() {
+        if (proxyConfFileForTests == null) {
+            // load the properties from the proxy.conf file
+            Properties properties = new Properties();
+            try (InputStream inputStream = new FileInputStream("../conf/proxy.conf")) {
+                properties.load(inputStream);
+            }
+            // set dummy values for the required properties so that validation is passed
+            properties.setProperty("brokerServiceURL", "pulsar://0.0.0.0:0");
+            properties.setProperty("brokerWebServiceURL", "http://0.0.0.0:0");
+            // change keepAliveIntervalSeconds default value so that it's possible to validate that it's configured
+            properties.setProperty("keepAliveIntervalSeconds", "25");
+            // write the properties to a temporary file
+            proxyConfFileForTests = File.createTempFile("proxy", ".conf");
+            proxyConfFileForTests.deleteOnExit();
+            try (OutputStream out = new FileOutputStream(proxyConfFileForTests)) {
+                properties.store(out, null);
+            }
+        }
+        return new String[] { "-c", proxyConfFileForTests.getAbsolutePath() };
+    }
 
     @Override
     @BeforeClass
     protected void setup() throws Exception {
         internalSetup();
-        serviceStarter = new ProxyServiceStarter(ARGS, null, true);
+        serviceStarter = new ProxyServiceStarter(getArgs(), null, true);
         serviceStarter.getConfig().setBrokerServiceURL(pulsar.getBrokerServiceUrl());
         serviceStarter.getConfig().setBrokerWebServiceURL(pulsar.getWebServiceAddress());
         serviceStarter.getConfig().setWebServicePort(Optional.of(0));
@@ -100,6 +128,11 @@ public class ProxyServiceStarterTest extends MockedPulsarServiceBaseTest {
         }
     }
 
+    @Test
+    public void testKeepAliveIntervalSecondsIsConfigured() throws Exception {
+        assertEquals(serviceStarter.getConfig().getKeepAliveIntervalSeconds(), 25);
+    }
+
     @Test
     public void testProduceAndConsumeMessageWithWebsocket() throws Exception {
         @Cleanup("stop")
@@ -180,7 +213,7 @@ public class ProxyServiceStarterTest extends MockedPulsarServiceBaseTest {
 
 
 
-        ProxyServiceStarter serviceStarter = new ProxyServiceStarter(ARGS, null, true);
+        ProxyServiceStarter serviceStarter = new ProxyServiceStarter(getArgs(), null, true);
         initConfig.accept(serviceStarter.getConfig());
         // ProxyServiceStarter will throw an exception when Authentication#start is failed
         serviceStarter.getConfig().setBrokerClientAuthenticationPlugin(ExceptionAuthentication1.class.getName());
@@ -192,7 +225,7 @@ public class ProxyServiceStarterTest extends MockedPulsarServiceBaseTest {
             assertTrue(serviceStarter.getProxyClientAuthentication() instanceof ExceptionAuthentication1);
         }
 
-        serviceStarter = new ProxyServiceStarter(ARGS, null, true);
+        serviceStarter = new ProxyServiceStarter(getArgs(), null, true);
         initConfig.accept(serviceStarter.getConfig());
         // ProxyServiceStarter will throw an exception when Authentication#start and Authentication#close are failed
         serviceStarter.getConfig().setBrokerClientAuthenticationPlugin(ExceptionAuthentication2.class.getName());
