@@ -19,11 +19,14 @@
 package org.apache.pulsar.testclient;
 
 import static org.apache.commons.lang3.StringUtils.isNotBlank;
+import io.opentelemetry.sdk.autoconfigure.AutoConfiguredOpenTelemetrySdk;
 import java.lang.management.ManagementFactory;
+import java.util.Map;
 import java.util.Objects;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Consumer;
 import lombok.experimental.UtilityClass;
+import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.io.FileUtils;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminBuilder;
@@ -37,6 +40,7 @@ import org.slf4j.Logger;
 /**
  * Utility for test clients.
  */
+@Slf4j
 @UtilityClass
 public class PerfClientUtils {
 
@@ -76,7 +80,12 @@ public class PerfClientUtils {
                 .listenerThreads(arguments.listenerThreads)
                 .tlsTrustCertsFilePath(arguments.tlsTrustCertsFilePath)
                 .maxLookupRequests(arguments.maxLookupRequest)
-                .proxyServiceUrl(arguments.proxyServiceURL, arguments.proxyProtocol);
+                .proxyServiceUrl(arguments.proxyServiceURL, arguments.proxyProtocol)
+                .openTelemetry(AutoConfiguredOpenTelemetrySdk.builder()
+                        .addPropertiesSupplier(() -> Map.of(
+                                "otel.sdk.disabled", "true"
+                        ))
+                        .build().getOpenTelemetrySdk());
 
         if (isNotBlank(arguments.authPluginClassName)) {
             clientBuilder.authentication(arguments.authPluginClassName, arguments.authParams);
@@ -119,4 +128,81 @@ public class PerfClientUtils {
         return pulsarAdminBuilder;
     }
 
+    /**
+     * This is used to register a shutdown hook that will be called when the JVM exits.
+     * @param runnable the runnable to run on shutdown
+     * @return the thread that was registered as a shutdown hook
+     */
+    public static Thread addShutdownHook(Runnable runnable) {
+        Thread shutdownHookThread = new Thread(runnable, "perf-client-shutdown");
+        Runtime.getRuntime().addShutdownHook(shutdownHookThread);
+        return shutdownHookThread;
+    }
+
+    /**
+     * This is used to remove a previously registered shutdown hook and run it immediately.
+     * This is useful at least for tests when there are multiple instances of the classes
+     * in the JVM. It will also prevent resource leaks when test code isn't relying on the JVM
+     * exit to clean up resources.
+     * @param shutdownHookThread the shutdown hook thread to remove and run
+     * @throws InterruptedException if the thread is interrupted while waiting for it to finish
+     */
+    public static void removeAndRunShutdownHook(Thread shutdownHookThread) throws InterruptedException {
+        // clear interrupted status and restore later
+        boolean wasInterrupted = Thread.currentThread().interrupted();
+        try {
+            Runtime.getRuntime().removeShutdownHook(shutdownHookThread);
+            shutdownHookThread.start();
+            shutdownHookThread.join();
+        } finally {
+            if (wasInterrupted) {
+                Thread.currentThread().interrupt();
+            }
+        }
+    }
+
+    /**
+     * This is used to close the client so that the interrupted status is cleared before
+     * closing the client. This is needed if the thread is already interrupted before calling this method.
+     * @param client the client to close
+     */
+    public static void closeClient(PulsarClient client) {
+        if (client == null) {
+            return;
+        }
+        // clear interrupted status so that the client can be shutdown
+        boolean wasInterrupted = Thread.currentThread().interrupted();
+        try {
+            client.close();
+        } catch (PulsarClientException e) {
+            log.error("Failed to close client", e);
+        } finally {
+            if (wasInterrupted) {
+                Thread.currentThread().interrupt();
+            }
+        }
+    }
+
+    /**
+     * Check if the throwable or any of its causes is an InterruptedException.
+     *
+     * @param throwable the throwable to check
+     * @return true if the throwable or any of its causes is an InterruptedException, false otherwise
+     */
+    public static boolean hasInterruptedException(Throwable throwable) {
+        if (throwable == null) {
+            return false;
+        }
+        if (throwable instanceof InterruptedException) {
+            return true;
+        }
+        Throwable cause = throwable.getCause();
+        while (cause != null) {
+            if (cause instanceof InterruptedException) {
+                return true;
+            }
+            cause = cause.getCause();
+        }
+        return false;
+    }
 }
