@@ -41,15 +41,12 @@ import org.apache.pulsar.client.api.ConsumerBuilder;
 import org.apache.pulsar.client.api.ConsumerCryptoFailureAction;
 import org.apache.pulsar.client.api.DeadLetterPolicy;
 import org.apache.pulsar.client.api.MessageId;
-import org.apache.pulsar.client.api.MessageIdAdv;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.PulsarClientException.AlreadyClosedException;
 import org.apache.pulsar.client.api.SubscriptionMode;
 import org.apache.pulsar.client.api.SubscriptionType;
-import org.apache.pulsar.client.api.TopicMessageId;
 import org.apache.pulsar.client.impl.ConsumerBuilderImpl;
-import org.apache.pulsar.client.impl.TopicMessageIdImpl;
 import org.apache.pulsar.common.policies.data.TopicOperation;
 import org.apache.pulsar.common.util.Codec;
 import org.apache.pulsar.common.util.DateFormatter;
@@ -75,7 +72,7 @@ import org.slf4j.LoggerFactory;
  */
 public class ConsumerHandler extends AbstractWebSocketHandler {
 
-    private String subscription = null;
+    protected String subscription = null;
     private SubscriptionType subscriptionType;
     private SubscriptionMode subscriptionMode;
     private Consumer<byte[]> consumer;
@@ -88,6 +85,10 @@ public class ConsumerHandler extends AbstractWebSocketHandler {
     private final LongAdder numBytesDelivered;
     private final LongAdder numMsgsAcked;
     private volatile long msgDeliveredCounter = 0;
+
+    protected String topicsPattern;
+
+    protected String topics;
     private static final AtomicLongFieldUpdater<ConsumerHandler> MSG_DELIVERED_COUNTER_UPDATER =
             AtomicLongFieldUpdater.newUpdater(ConsumerHandler.class, "msgDeliveredCounter");
 
@@ -123,7 +124,14 @@ public class ConsumerHandler extends AbstractWebSocketHandler {
                 return;
             }
 
-            this.consumer = builder.topic(topic.toString()).subscriptionName(subscription).subscribe();
+            if (topicsPattern != null) {
+                this.consumer = builder.topicsPattern(topicsPattern).subscriptionName(subscription).subscribe();
+            } else if (topics != null) {
+                this.consumer = builder.topics(Splitter.on(",").splitToList(topics))
+                        .subscriptionName(subscription).subscribe();
+            } else {
+                this.consumer = builder.topic(topic.toString()).subscriptionName(subscription).subscribe();
+            }
             if (!this.service.addConsumer(this)) {
                 log.warn("[{}:{}] Failed to add consumer handler for topic {}", request.getRemoteAddr(),
                         request.getRemotePort(), topic);
@@ -299,8 +307,7 @@ public class ConsumerHandler extends AbstractWebSocketHandler {
 
     private void handleAck(ConsumerCommand command) throws IOException {
         // We should have received an ack
-        TopicMessageId msgId = new TopicMessageIdImpl(topic.toString(),
-                (MessageIdAdv) MessageId.fromByteArray(Base64.getDecoder().decode(command.messageId)));
+        MessageId msgId = MessageId.fromByteArray(Base64.getDecoder().decode(command.messageId));
         if (log.isDebugEnabled()) {
             log.debug("[{}/{}] Received ack request of message {} from {} ", consumer.getTopic(),
                     subscription, msgId, getRemote().getInetSocketAddress().toString());
@@ -465,6 +472,8 @@ public class ConsumerHandler extends AbstractWebSocketHandler {
 
         if (service.getCryptoKeyReader().isPresent()) {
             builder.cryptoKeyReader(service.getCryptoKeyReader().get());
+        } else {
+            // If users want to decrypt messages themselves, they should set "cryptoFailureAction" to "CONSUME".
         }
         return builder;
     }
@@ -488,7 +497,7 @@ public class ConsumerHandler extends AbstractWebSocketHandler {
         }
     }
 
-    public static String extractSubscription(HttpServletRequest request) {
+    public String extractSubscription(HttpServletRequest request) {
         String uri = request.getRequestURI();
         List<String> parts = Splitter.on("/").splitToList(uri);
 
