@@ -21,7 +21,7 @@ package org.apache.pulsar.tests.integration.backwardscompatibility;
 import java.util.function.Supplier;
 import org.testng.annotations.Test;
 
-public class SmokeTest2_3 extends PulsarStandaloneTestSuite2_3 {
+public class SmokeTest2_5 extends PulsarStandaloneTestSuite2_5 {
 
     @Test(dataProvider = "StandaloneServiceUrlAndTopics")
     public void testPublishAndConsume(Supplier<String> serviceUrl, boolean isPersistent) throws Exception {
@@ -32,4 +32,9 @@ public class SmokeTest2_3 extends PulsarStandaloneTestSuite2_3 {
     public void testBatchMessagePublishAndConsume(Supplier<String> serviceUrl, boolean isPersistent) throws Exception {
         super.testBatchMessagePublishAndConsume(serviceUrl.get(), isPersistent);
     }
+
+    @Test(dataProvider = "StandaloneServiceUrlAndTopics")
+    public void testBatchIndexAckDisabled(Supplier<String> serviceUrl, boolean isPersistent) throws Exception {
+        super.testBatchIndexAckDisabled(serviceUrl.get());
+    }
 }
