@@ -86,13 +86,14 @@ public class ClientToolTest extends TopicMessagingBase {
                     + "\nError output:\n" + result.getStderr());
         }
         String output = result.getStdout();
-        Pattern message = Pattern.compile("----- got message -----\nkey:\\[null\\], properties:\\[\\], content:(.*)");
+        Pattern message = Pattern.compile(
+                "----- got message -----\npublishTime:\\[(.*)\\], eventTime:\\[(.*)\\], key:\\[null\\], "
+                        + "properties:\\[\\], content:(.*)");
         Matcher matcher = message.matcher(output);
         List<String> received = new ArrayList<>(MESSAGE_COUNT);
         while (matcher.find()) {
-            received.add(matcher.group(1));
+            received.add(matcher.group(3));
         }
         return received;
     }
-
 }
