@@ -21,7 +21,7 @@ package org.apache.pulsar.tests.integration.containers;
 /**
  * A pulsar container that runs configuration store.
  */
-public class CSContainer extends ZKContainer<CSContainer> {
+public class CSContainer extends PulsarContainer<CSContainer> {
 
     public static final String NAME = "configuration-store";
 
@@ -34,4 +34,9 @@ public class CSContainer extends ZKContainer<CSContainer> {
             CS_PORT,
             INVALID_PORT);
     }
+
+    @Override
+    protected boolean isCodeCoverageEnabled() {
+        return false;
+    }
 }
