@@ -22,6 +22,7 @@ import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 
+import dev.failsafe.Failsafe;
 import java.util.Map;
 
 import org.apache.commons.lang3.StringUtils;
@@ -45,7 +46,6 @@ import com.google.gson.Gson;
 
 import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
-import net.jodah.failsafe.Failsafe;
 
 @Slf4j
 public class PulsarIOSourceRunner extends PulsarIOTestRunner {
