@@ -35,6 +35,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.function.Function;
+import lombok.Cleanup;
 import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.io.IOUtils;
@@ -44,9 +45,11 @@ import org.apache.pulsar.tests.integration.containers.BrokerContainer;
 import org.apache.pulsar.tests.integration.containers.CSContainer;
 import org.apache.pulsar.tests.integration.containers.ProxyContainer;
 import org.apache.pulsar.tests.integration.containers.PulsarContainer;
+import org.apache.pulsar.tests.integration.containers.PulsarInitMetadataContainer;
 import org.apache.pulsar.tests.integration.containers.WorkerContainer;
 import org.apache.pulsar.tests.integration.containers.ZKContainer;
 import org.apache.pulsar.tests.integration.docker.ContainerExecResult;
+import org.apache.pulsar.tests.integration.oxia.OxiaContainer;
 import org.testcontainers.containers.BindMode;
 import org.testcontainers.containers.GenericContainer;
 import org.testcontainers.containers.Network;
@@ -69,9 +72,12 @@ public class PulsarCluster {
      * @return the built pulsar cluster
      */
     public static PulsarCluster forSpec(PulsarClusterSpec spec) {
-        CSContainer csContainer = new CSContainer(spec.clusterName)
-                .withNetwork(Network.newNetwork())
-                .withNetworkAliases(CSContainer.NAME);
+        CSContainer csContainer = null;
+        if (!spec.enableOxia) {
+            csContainer = new CSContainer(spec.clusterName)
+                    .withNetwork(Network.newNetwork())
+                    .withNetworkAliases(CSContainer.NAME);
+        }
         return new PulsarCluster(spec, csContainer, false);
     }
 
@@ -86,6 +92,8 @@ public class PulsarCluster {
     private final String clusterName;
     private final Network network;
     private final ZKContainer zkContainer;
+
+    private final OxiaContainer oxiaContainer;
     private final CSContainer csContainer;
     private final boolean sharedCsContainer;
     private final Map<String, BKContainer> bookieContainers;
@@ -94,23 +102,47 @@ public class PulsarCluster {
     private final ProxyContainer proxyContainer;
     private Map<String, GenericContainer<?>> externalServices = Collections.emptyMap();
     private Map<String, Map<String, String>> externalServiceEnvs;
+    private final Map<String, String> functionWorkerEnvs;
+    private final List<Integer> functionWorkerAdditionalPorts;
+
+    private final String metadataStoreUrl;
+    private final String configurationMetadataStoreUrl;
 
     private PulsarCluster(PulsarClusterSpec spec, CSContainer csContainer, boolean sharedCsContainer) {
 
         this.spec = spec;
         this.sharedCsContainer = sharedCsContainer;
         this.clusterName = spec.clusterName();
-        this.network = csContainer.getNetwork();
+        if (csContainer != null ) {
+            this.network = csContainer.getNetwork();
+        } else {
+            this.network = Network.newNetwork();
+        }
 
-        this.zkContainer = new ZKContainer(clusterName);
-        this.zkContainer
-            .withNetwork(network)
-            .withNetworkAliases(appendClusterName(ZKContainer.NAME))
-            .withEnv("clusterName", clusterName)
-            .withEnv("zkServers", appendClusterName(ZKContainer.NAME))
-            .withEnv("configurationStore", CSContainer.NAME + ":" + CS_PORT)
-            .withEnv("forceSync", "no")
-            .withEnv("pulsarNode", appendClusterName("pulsar-broker-0"));
+
+
+        if (spec.enableOxia) {
+            this.zkContainer = null;
+            this.oxiaContainer = new OxiaContainer(clusterName);
+            this.oxiaContainer
+                    .withNetwork(network)
+                    .withNetworkAliases(appendClusterName(OxiaContainer.NAME));
+            metadataStoreUrl = "oxia://" + oxiaContainer.getServiceAddress();
+            configurationMetadataStoreUrl = metadataStoreUrl;
+        } else {
+            this.oxiaContainer = null;
+            this.zkContainer = new ZKContainer(clusterName);
+            this.zkContainer
+                    .withNetwork(network)
+                    .withNetworkAliases(appendClusterName(ZKContainer.NAME))
+                    .withEnv("clusterName", clusterName)
+                    .withEnv("zkServers", appendClusterName(ZKContainer.NAME))
+                    .withEnv("configurationStore", CSContainer.NAME + ":" + CS_PORT)
+                    .withEnv("forceSync", "no")
+                    .withEnv("pulsarNode", appendClusterName("pulsar-broker-0"));
+            metadataStoreUrl = appendClusterName(ZKContainer.NAME);
+            configurationMetadataStoreUrl = CSContainer.NAME + ":" + CS_PORT;
+        }
 
         this.csContainer = csContainer;
 
@@ -121,27 +153,29 @@ public class PulsarCluster {
         this.proxyContainer = new ProxyContainer(clusterName, appendClusterName(ProxyContainer.NAME), spec.enableTls)
                 .withNetwork(network)
                 .withNetworkAliases(appendClusterName("pulsar-proxy"))
-                .withEnv("zkServers", appendClusterName(ZKContainer.NAME))
-                .withEnv("zookeeperServers", appendClusterName(ZKContainer.NAME))
-                .withEnv("configurationStoreServers", CSContainer.NAME + ":" + CS_PORT)
+                .withEnv("metadataStoreUrl", metadataStoreUrl)
+                .withEnv("configurationMetadataStoreUrl", configurationMetadataStoreUrl)
                 .withEnv("clusterName", clusterName);
-                // enable mTLS
+        // enable mTLS
         if (spec.enableTls) {
             proxyContainer
-                .withEnv("webServicePortTls", String.valueOf(BROKER_HTTPS_PORT))
-                .withEnv("servicePortTls", String.valueOf(BROKER_PORT_TLS))
-                .withEnv("forwardAuthorizationCredentials", "true")
-                .withEnv("tlsRequireTrustedClientCertOnConnect", "true")
-                .withEnv("tlsAllowInsecureConnection", "false")
-                .withEnv("tlsCertificateFilePath", "/pulsar/certificate-authority/server-keys/proxy.cert.pem")
-                .withEnv("tlsKeyFilePath", "/pulsar/certificate-authority/server-keys/proxy.key-pk8.pem")
-                .withEnv("tlsTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca.cert.pem")
-                .withEnv("brokerClientAuthenticationPlugin", AuthenticationTls.class.getName())
-                .withEnv("brokerClientAuthenticationParameters", String.format("tlsCertFile:%s,tlsKeyFile:%s", "/pulsar/certificate-authority/client-keys/admin.cert.pem", "/pulsar/certificate-authority/client-keys/admin.key-pk8.pem"))
-                .withEnv("tlsEnabledWithBroker", "true")
-                .withEnv("brokerClientTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca.cert.pem")
-                .withEnv("brokerClientCertificateFilePath", "/pulsar/certificate-authority/server-keys/proxy.cert.pem")
-                .withEnv("brokerClientKeyFilePath", "/pulsar/certificate-authority/server-keys/proxy.key-pk8.pem");
+                    .withEnv("webServicePortTls", String.valueOf(BROKER_HTTPS_PORT))
+                    .withEnv("servicePortTls", String.valueOf(BROKER_PORT_TLS))
+                    .withEnv("forwardAuthorizationCredentials", "true")
+                    .withEnv("tlsRequireTrustedClientCertOnConnect", "true")
+                    .withEnv("tlsAllowInsecureConnection", "false")
+                    .withEnv("tlsCertificateFilePath", "/pulsar/certificate-authority/server-keys/proxy.cert.pem")
+                    .withEnv("tlsKeyFilePath", "/pulsar/certificate-authority/server-keys/proxy.key-pk8.pem")
+                    .withEnv("tlsTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca.cert.pem")
+                    .withEnv("brokerClientAuthenticationPlugin", AuthenticationTls.class.getName())
+                    .withEnv("brokerClientAuthenticationParameters", String.format("tlsCertFile:%s,tlsKeyFile:%s",
+                            "/pulsar/certificate-authority/client-keys/admin.cert.pem",
+                            "/pulsar/certificate-authority/client-keys/admin.key-pk8.pem"))
+                    .withEnv("tlsEnabledWithBroker", "true")
+                    .withEnv("brokerClientTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca.cert.pem")
+                    .withEnv("brokerClientCertificateFilePath",
+                            "/pulsar/certificate-authority/server-keys/proxy.cert.pem")
+                    .withEnv("brokerClientKeyFilePath", "/pulsar/certificate-authority/server-keys/proxy.key-pk8.pem");
 
         }
         if (spec.proxyEnvs != null) {
@@ -150,6 +184,9 @@ public class PulsarCluster {
         if (spec.proxyMountFiles != null) {
             spec.proxyMountFiles.forEach(this.proxyContainer::withFileSystemBind);
         }
+        if (spec.proxyAdditionalPorts != null) {
+            spec.proxyAdditionalPorts.forEach(this.proxyContainer::addExposedPort);
+        }
 
         // create bookies
         bookieContainers.putAll(
@@ -157,7 +194,7 @@ public class PulsarCluster {
                     BKContainer bookieContainer = new BKContainer(clusterName, name)
                             .withNetwork(network)
                             .withNetworkAliases(appendClusterName(name))
-                            .withEnv("zkServers", appendClusterName(ZKContainer.NAME))
+                            .withEnv("metadataServiceUri", "metadata-store:" + metadataStoreUrl)
                             .withEnv("useHostNameAsBookieID", "true")
                             // Disable fsyncs for tests since they're slow within the containers
                             .withEnv("journalSyncData", "false")
@@ -179,50 +216,56 @@ public class PulsarCluster {
 
         // create brokers
         brokerContainers.putAll(
-            runNumContainers("broker", spec.numBrokers(), (name) -> {
-                BrokerContainer brokerContainer = new BrokerContainer(clusterName, appendClusterName(name), spec.enableTls)
-                        .withNetwork(network)
-                        .withNetworkAliases(appendClusterName(name))
-                        .withEnv("zkServers", appendClusterName(ZKContainer.NAME))
-                        .withEnv("zookeeperServers", appendClusterName(ZKContainer.NAME))
-                        .withEnv("configurationStoreServers", CSContainer.NAME + ":" + CS_PORT)
-                        .withEnv("clusterName", clusterName)
-                        .withEnv("brokerServiceCompactionMonitorIntervalInSeconds", "1")
-                        .withEnv("loadBalancerOverrideBrokerNicSpeedGbps", "1")
-                        // used in s3 tests
-                        .withEnv("AWS_ACCESS_KEY_ID", "accesskey").withEnv("AWS_SECRET_KEY", "secretkey")
-                        .withEnv("maxMessageSize", "" + spec.maxMessageSize);
-                    if (spec.enableTls) {
-                        // enable mTLS
-                        brokerContainer
-                            .withEnv("webServicePortTls", String.valueOf(BROKER_HTTPS_PORT))
-                            .withEnv("brokerServicePortTls", String.valueOf(BROKER_PORT_TLS))
-                            .withEnv("authenticateOriginalAuthData", "true")
-                            .withEnv("tlsAllowInsecureConnection", "false")
-                            .withEnv("tlsRequireTrustedClientCertOnConnect", "true")
-                            .withEnv("tlsTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca.cert.pem")
-                            .withEnv("tlsCertificateFilePath", "/pulsar/certificate-authority/server-keys/broker.cert.pem")
-                            .withEnv("tlsKeyFilePath", "/pulsar/certificate-authority/server-keys/broker.key-pk8.pem");
-                    }
-                    if (spec.queryLastMessage) {
-                        brokerContainer.withEnv("bookkeeperExplicitLacIntervalInMills", "10");
-                        brokerContainer.withEnv("bookkeeperUseV2WireProtocol", "false");
-                    }
-                    if (spec.brokerEnvs != null) {
-                        brokerContainer.withEnv(spec.brokerEnvs);
-                    }
-                    if (spec.brokerMountFiles != null) {
-                        spec.brokerMountFiles.forEach(brokerContainer::withFileSystemBind);
-                    }
-                    if (spec.brokerAdditionalPorts() != null) {
-                        spec.brokerAdditionalPorts().forEach(brokerContainer::addExposedPort);
-                    }
-                    return brokerContainer;
-                }
-            ));
+                runNumContainers("broker", spec.numBrokers(), (name) -> {
+                            BrokerContainer brokerContainer =
+                                    new BrokerContainer(clusterName, appendClusterName(name), spec.enableTls)
+                                            .withNetwork(network)
+                                            .withNetworkAliases(appendClusterName(name))
+                                            .withEnv("metadataStoreUrl", metadataStoreUrl)
+                                            .withEnv("configurationMetadataStoreUrl", configurationMetadataStoreUrl)
+                                            .withEnv("clusterName", clusterName)
+                                            .withEnv("brokerServiceCompactionMonitorIntervalInSeconds", "1")
+                                            .withEnv("loadBalancerOverrideBrokerNicSpeedGbps", "1")
+                                            // used in s3 tests
+                                            .withEnv("AWS_ACCESS_KEY_ID", "accesskey").withEnv("AWS_SECRET_KEY",
+                                                    "secretkey")
+                                            .withEnv("maxMessageSize", "" + spec.maxMessageSize);
+                            if (spec.enableTls) {
+                                // enable mTLS
+                                brokerContainer
+                                        .withEnv("webServicePortTls", String.valueOf(BROKER_HTTPS_PORT))
+                                        .withEnv("brokerServicePortTls", String.valueOf(BROKER_PORT_TLS))
+                                        .withEnv("authenticateOriginalAuthData", "true")
+                                        .withEnv("tlsAllowInsecureConnection", "false")
+                                        .withEnv("tlsRequireTrustedClientCertOnConnect", "true")
+                                        .withEnv("tlsTrustCertsFilePath", "/pulsar/certificate-authority/certs/ca"
+                                                + ".cert.pem")
+                                        .withEnv("tlsCertificateFilePath",
+                                                "/pulsar/certificate-authority/server-keys/broker.cert.pem")
+                                        .withEnv("tlsKeyFilePath",
+                                                "/pulsar/certificate-authority/server-keys/broker.key-pk8.pem");
+                            }
+                            if (spec.queryLastMessage) {
+                                brokerContainer.withEnv("bookkeeperExplicitLacIntervalInMills", "10");
+                                brokerContainer.withEnv("bookkeeperUseV2WireProtocol", "false");
+                            }
+                            if (spec.brokerEnvs != null) {
+                                brokerContainer.withEnv(spec.brokerEnvs);
+                            }
+                            if (spec.brokerMountFiles != null) {
+                                spec.brokerMountFiles.forEach(brokerContainer::withFileSystemBind);
+                            }
+                            if (spec.brokerAdditionalPorts() != null) {
+                                spec.brokerAdditionalPorts().forEach(brokerContainer::addExposedPort);
+                            }
+                            return brokerContainer;
+                        }
+                ));
 
         spec.classPathVolumeMounts.forEach((key, value) -> {
-            zkContainer.withClasspathResourceMapping(key, value, BindMode.READ_WRITE);
+            if (zkContainer != null) {
+                zkContainer.withClasspathResourceMapping(key, value, BindMode.READ_WRITE);
+            }
             proxyContainer.withClasspathResourceMapping(key, value, BindMode.READ_WRITE);
 
             bookieContainers.values().forEach(c -> c.withClasspathResourceMapping(key, value, BindMode.READ_WRITE));
@@ -230,6 +273,8 @@ public class PulsarCluster {
             workerContainers.values().forEach(c -> c.withClasspathResourceMapping(key, value, BindMode.READ_WRITE));
         });
 
+        functionWorkerEnvs = spec.functionWorkerEnvs;
+        functionWorkerAdditionalPorts = spec.functionWorkerAdditionalPorts;
     }
 
     public String getPlainTextServiceUrl() {
@@ -278,20 +323,33 @@ public class PulsarCluster {
     }
 
     public void start() throws Exception {
-        // start the local zookeeper
-        zkContainer.start();
-        log.info("Successfully started local zookeeper container.");
 
-        // start the configuration store
-        if (!sharedCsContainer) {
-            csContainer.start();
-            log.info("Successfully started configuration store container.");
+        if (!spec.enableOxia) {
+            // start the local zookeeper
+            zkContainer.start();
+            log.info("Successfully started local zookeeper container.");
+
+            // start the configuration store
+            if (!sharedCsContainer) {
+                csContainer.start();
+                log.info("Successfully started configuration store container.");
+            }
+        } else {
+            oxiaContainer.start();
         }
 
-        // init the cluster
-        zkContainer.execCmd(
-            "bin/init-cluster.sh");
-        log.info("Successfully initialized the cluster.");
+        {
+            // Run cluster metadata initialization
+            @Cleanup
+            PulsarInitMetadataContainer init = new PulsarInitMetadataContainer(
+                    network,
+                    clusterName,
+                    metadataStoreUrl,
+                    configurationMetadataStoreUrl,
+                    appendClusterName("pulsar-broker-0")
+            );
+            init.initialize();
+        }
 
         // start bookies
         bookieContainers.values().forEach(BKContainer::start);
@@ -318,7 +376,8 @@ public class PulsarCluster {
                 serviceContainer.withNetwork(network);
                 serviceContainer.withNetworkAliases(service.getKey());
                 if (null != externalServiceEnvs && null != externalServiceEnvs.get(service.getKey())) {
-                    Map<String, String> env = externalServiceEnvs.getOrDefault(service.getKey(), Collections.emptyMap());
+                    Map<String, String> env =
+                            externalServiceEnvs.getOrDefault(service.getKey(), Collections.emptyMap());
                     serviceContainer.withEnv(env);
                 }
                 PulsarContainer.configureLeaveContainerRunning(serviceContainer);
@@ -390,6 +449,10 @@ public class PulsarCluster {
             zkContainer.stop();
         }
 
+        if (oxiaContainer != null) {
+            oxiaContainer.stop();
+        }
+
         try {
             network.close();
         } catch (Exception e) {
@@ -403,7 +466,8 @@ public class PulsarCluster {
                 .forEach(GenericContainer::stop);
     }
 
-    public synchronized void setupFunctionWorkers(String suffix, FunctionRuntimeType runtimeType, int numFunctionWorkers) {
+    public synchronized void setupFunctionWorkers(String suffix, FunctionRuntimeType runtimeType,
+                                                  int numFunctionWorkers) {
         switch (runtimeType) {
             case THREAD:
                 startFunctionWorkersWithThreadContainerFactory(suffix, numFunctionWorkers);
@@ -440,7 +504,9 @@ public class PulsarCluster {
                 .withEnv("clusterName", clusterName)
                 .withEnv("zookeeperServers", ZKContainer.NAME)
                 // bookkeeper tools
-                .withEnv("zkServers", ZKContainer.NAME);
+                .withEnv("zkServers", ZKContainer.NAME)
+                .withEnv(functionWorkerEnvs)
+                .withExposedPorts(functionWorkerAdditionalPorts.toArray(new Integer[0]));
     }
 
     private void startFunctionWorkersWithThreadContainerFactory(String suffix, int numFunctionWorkers) {
@@ -491,9 +557,9 @@ public class PulsarCluster {
         containers.forEach((name, container) -> {
             PulsarContainer.configureLeaveContainerRunning(container);
             container
-                .withNetwork(network)
-                .withNetworkAliases(name)
-                .start();
+                    .withNetwork(network)
+                    .withNetworkAliases(name)
+                    .start();
             log.info("Successfully start container {}.", name);
         });
     }
@@ -565,15 +631,16 @@ public class PulsarCluster {
         return zkContainer;
     }
 
-    public ContainerExecResult runAdminCommandOnAnyBroker(String...commands) throws Exception {
+    public ContainerExecResult runAdminCommandOnAnyBroker(String... commands) throws Exception {
         return runCommandOnAnyBrokerWithScript(ADMIN_SCRIPT, commands);
     }
 
-    public ContainerExecResult runPulsarBaseCommandOnAnyBroker(String...commands) throws Exception {
+    public ContainerExecResult runPulsarBaseCommandOnAnyBroker(String... commands) throws Exception {
         return runCommandOnAnyBrokerWithScript(PULSAR_COMMAND_SCRIPT, commands);
     }
 
-    private ContainerExecResult runCommandOnAnyBrokerWithScript(String scriptType, String...commands) throws Exception {
+    private ContainerExecResult runCommandOnAnyBrokerWithScript(String scriptType, String... commands)
+            throws Exception {
         BrokerContainer container = getAnyBroker();
         String[] cmds = new String[commands.length + 1];
         cmds[0] = scriptType;
@@ -607,8 +674,8 @@ public class PulsarCluster {
 
     public ContainerExecResult createNamespace(String nsName) throws Exception {
         return runAdminCommandOnAnyBroker(
-            "namespaces", "create", "public/" + nsName,
-            "--clusters", clusterName);
+                "namespaces", "create", "public/" + nsName,
+                "--clusters", clusterName);
     }
 
     public ContainerExecResult createPartitionedTopic(String topicName, int partitions) throws Exception {
@@ -619,8 +686,8 @@ public class PulsarCluster {
 
     public ContainerExecResult enableDeduplication(String nsName, boolean enabled) throws Exception {
         return runAdminCommandOnAnyBroker(
-            "namespaces", "set-deduplication", "public/" + nsName,
-            enabled ? "--enable" : "--disable");
+                "namespaces", "set-deduplication", "public/" + nsName,
+                enabled ? "--enable" : "--disable");
     }
 
     public void dumpFunctionLogs(String name) {
@@ -633,7 +700,8 @@ public class PulsarCluster {
                 });
                 log.info("Function {} logs {}", name, logs);
             } catch (com.github.dockerjava.api.exception.NotFoundException notFound) {
-                log.info("Cannot download {} logs from {} not found exception {}", name, container.getContainerName(), notFound.toString());
+                log.info("Cannot download {} logs from {} not found exception {}", name, container.getContainerName(),
+                        notFound.toString());
             } catch (Throwable err) {
                 log.info("Cannot download {} logs from {}", name, container.getContainerName(), err);
             }
