@@ -147,6 +147,12 @@ public class PulsarClusterSpec {
      */
     Map<String, String> bookkeeperEnvs;
 
+    /**
+     * Specify envs for function workers.
+     */
+    @Singular
+    Map<String, String> functionWorkerEnvs;
+
     /**
      * Specify mount files.
      */
@@ -170,9 +176,23 @@ public class PulsarClusterSpec {
      */
     List<Integer> bookieAdditionalPorts;
 
+    /**
+     * Additional ports to expose on proxy containers.
+     */
+    List<Integer> proxyAdditionalPorts;
+
+    /**
+     * Additional ports to expose on function workers.
+     */
+    @Singular
+    List<Integer> functionWorkerAdditionalPorts;
+
     /**
      * Enable TLS for connection.
      */
     @Default
     boolean enableTls = false;
+
+    @Default
+    boolean enableOxia = false;
 }
