@@ -20,6 +20,7 @@ package org.apache.bookkeeper.mledger.offload.jcloud.impl;
 
 import io.netty.buffer.ByteBuf;
 import java.util.Map;
+import java.util.NavigableMap;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
 import java.util.function.Predicate;
@@ -274,7 +275,7 @@ public class MockManagedLedger implements ManagedLedger {
 
     @Override
     public ManagedLedgerConfig getConfig() {
-        return null;
+        return new ManagedLedgerConfig();
     }
 
     @Override
@@ -381,6 +382,11 @@ public class MockManagedLedger implements ManagedLedger {
         // no-op
     }
 
+    @Override
+    public NavigableMap<Long, LedgerInfo> getLedgersInfo() {
+        return null;
+    }
+
     @Override
     public CompletableFuture<Position> asyncMigrate() {
         // no-op
